-- Covert Operations Globals
-- Author: sman
-- DateCreated: 02/05/18 07:51:38 AM
--------------------------------------------------------------

-- =================================================================================
--			Functions

function SetUpGlobals ()
	
	-- ========================================================================================================================
	-- Human player global adjustments, based on game difficulty, etc.					

	g_Human_Minimum_Turn_Interval = math.floor( (g_iHandicap * 1.5) + 2 )						-- Human minimum launch window of 12 turns on deity, 8 turns on king, and 2 turns on settler 
	local iGameSpeedDivisor = g_iGameSpeed + 1
	g_Human_Minimum_Turn_Interval = math.floor(g_Human_Minimum_Turn_Interval / iGameSpeedDivisor)	-- Should be between ".5" (Settler/Quick, adjusted later) and 12 (Diety/Marathon)
	
	if g_Human_Minimum_Turn_Interval > 15 then g_Human_Minimum_Turn_Interval = 15 end		-- Add upper bound in case another mod adds a new difficulty level
	if g_Human_Minimum_Turn_Interval < 3 then g_Human_Minimum_Turn_Interval = 3 end			-- Add lower bound in case another mod adds a new difficulty level (for immortal, deity)
	
	g_Human_Last_Turn_CO_Launched = Game.GetGameTurn() or 0									-- Set starting game turn as the latest turn CO launched - for saved games. Use 0 if all else fails.
	g_Human_Next_CO_Launch_Turn = g_Human_Last_Turn_CO_Launched + g_Human_Minimum_Turn_Interval + math.random(1, 3)

	-- TEMP ADJUSTMENT AREA
	--g_Human_Next_CO_Launch_Turn = Game.GetGameTurn() + 1									-- Reserved area where changes are made to force/test specific game mechanics
	--Players[0]:ChangeNumResourceTotal(RESOURCE_URANIUM_ID, 10)							-- Be sure to disable in final mod versions!
	-- END OF TEM PADJUSTMENT AREA

	print("Next human player CO turn: " .. g_Human_Next_CO_Launch_Turn)
		
	-- ========================================================================================================================
	-- AI global adjustments, based on game difficulty, etc.					
		
	g_Open_Mission_Limit = (4 - g_iGameSpeed) + 1											-- AI can have 2 missions open on quick, 3 on std, 4 on epic, and 5 on marathon 
	if g_Open_Mission_Limit > 5 then g_Open_Mission_Limit = 5 end							-- Add upper bound in case another mod adds a new gamespeed
	if g_Open_Mission_Limit < 1 then g_Open_Mission_Limit = 1 end							-- Add lower bound in case another mod adds a new gamespeed

	g_AI_Minimum_Turn_Interval = (8 - g_iHandicap) * 2 + 2									-- AI minimum launch window of 4 turns on deity, 10 turns on king, and 18 turns on settler (nerfed with the '+2' sman:20180215)
	if g_AI_Minimum_Turn_Interval > 15 then g_AI_Minimum_Turn_Interval = 15 end				-- Add upper bound in case another mod adds a new difficulty level
	if g_AI_Minimum_Turn_Interval < 3 then g_AI_Minimum_Turn_Interval = 3 end				-- Add lower bound in case another mod adds a new difficulty level (for immortal, deity)

	--print("Human GT interval: " .. g_Human_Minimum_Turn_Interval .. "     Last human CO game turn: " .. g_Human_Last_Turn_CO_Launched .. "    Open Mission Limit: " .. g_Open_Mission_Limit .. "    Min Turn Interval: " .. g_AI_Minimum_Turn_Interval)		
end
Events.SequenceGameInitComplete.Add( SetUpGlobals )


function PopulateLeaderStats()																-- Populate global table g_Leader_Stats at game start
	-- Add adjustments for leader stats: VictoryCompetitiveness, Boldness, Warmongerhate, Meanness; Major biases: MAJOR_CIV_APPROACH_WAR, MAJOR_CIV_APPROACH_AFRAID; Flavors: FLAVOR_ESPIONAGE, FLAVOR_OFFENSE, FLAVOR_DEFENSE
	--g_Starting_Game_Turn = Game.GetGameTurn()												-- Saves the initial game turn - in case this is a loaded save game; IOW: if the value isn't 0, then this is a loaded savegame		

	for iPlayer=1, GameDefines.MAX_MAJOR_CIVS-1 do											-- Cycle through all major (non-human) players - Get all civs, even ones no longer in the game (for savegames) - in case of resurrection		
		local iLeaderType = Players[iPlayer]:GetLeaderType()								-- Grab Leader data from 3 sources: Leader definition file, Leader_MajorCivApproachBiases, and Leader_Flavors
		local iCurrentEra = Players[iPlayer]:GetCurrentEra()
		local pLeaderInfo = GameInfo.Leaders[iLeaderType]
		local sLeaderName = GameInfo.Leaders[iLeaderType].Description or "unk"
		local iVicCompete = GameInfo.Leaders[iLeaderType].VictoryCompetitiveness or 3		-- The 3 or 5 used here is for a "safe" default, in case for some bizzarre reason we can't get a better number
		local iBoldness = GameInfo.Leaders[iLeaderType].Boldness or 3
		local iWarmongerHate = GameInfo.Leaders[iLeaderType].WarmongerHate or 3
		local iMeanness = GameInfo.Leaders[iLeaderType].Meanness or 3
		local iMajorWarApproach = 5
		local iMajorAfraidApproach = 5
		local iFlavorEspionage = 3
		local iFlavorOffense = 3
		local iFlavorDefense = 3

		for row in DB.Query("SELECT MajorCivApproachType, Bias FROM Leader_MajorCivApproachBiases WHERE LeaderType = '" .. pLeaderInfo.Type .."'" ) do	-- Get Major civ biases for this leader
			if row.MajorCivApproachType == "MAJOR_CIV_APPROACH_WAR" then
					iMajorWarApproach = row.Bias
			elseif row.MajorCivApproachType == "MAJOR_CIV_APPROACH_AFRAID" then
				iMajorAfraidApproach = row.Bias
			end			
		end
			
		for row in DB.Query("SELECT FlavorType, Flavor FROM Leader_Flavors WHERE LeaderType = '" .. pLeaderInfo.Type .."'" ) do		-- Get appropriate Flavors for this leader
			if row.FlavorType == "FLAVOR_ESPIONAGE" then
				iFlavorEspionage = row.Flavor
			elseif row.FlavorType == "FLAVOR_OFFENSE" then
				iFlavorOffense = row.Flavor
			elseif row.FlavorType == "FLAVOR_DEFENSE" then
				iFlavorDefense = row.Flavor
			end			
		end					
		
		-- the COVERT_OPERATIONS_PROPENSITY determines how aggressive this leader will use Covert Ops.  It adjusted based on the weighting for each variable
		local iCOP = math.floor(  ((iVicCompete * 2) + (iBoldness*2) + (iWarmongerHate*1) + (iMeanness*2) + (iMajorWarApproach*2) + (iFlavorEspionage*6) + (iFlavorOffense*3) -
									(iMajorAfraidApproach*3) - (iFlavorDefense*1)) * .33  )	 
										
		iCOP = iCOP + ( 25 - math.random(1, 50) )											-- Add +-25 to the iCOP - to provide some variability in leader behavior (sman:20180215)
		if iCOP < 1 then iCOP = 1 end

		g_Leader_Stats[iPlayer] = { [COVERT_OPERATIONS_PROPENSITY] = iCOP }					-- Set the COP value into the leader table
						
		if g_Starting_Game_Turn > 10 then													-- If initial game turn is over 10, then...  If so, then this is a loaded saved game - need to refresh the mission table
			Component1_Schedule_Mission(Players[iPlayer], g_Starting_Game_Turn, iCurrentEra)	-- Determine if iPlayer can schedule an attack planned for this game turn
		end
	end

	g_Leader_Stats[99] = { [COVERT_OPERATIONS_PROPENSITY] = 0 }								-- Will be the human player's COP, during Live Tuner runs?

end
Events.SequenceGameInitComplete.Add( PopulateLeaderStats )