-- Covert Operartions 20_Process_Mission_Outcomes.lua     MAIN/TOP-LEVEL FUNCTION
-- Author: sman
-- DateCreated: 01/08/18 9:18:23 AM
--------------------------------------------------------------

-- ===============================================================================================
--		Globals


-- ===============================================================================================
--		includes


-- ===============================================================================================
--		Support Functions

function GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)							-- Determines if the covert mission was successful - returns YES/NO
	local iMissionResult = NO																-- Default result is mission failed	
	local pTargetPlayer = Players[pTargetCity:GetOwner()]
	local iAttackerEspionagePosture = CalculateEspionagePosture(pPlayer, pTargetCity)		-- Get the EP for the attacker in target city
	local iDefenderEspionagePosture = CalculateEspionagePosture(pTargetPlayer, pTargetCity)	-- Get the EP for the defender in target city
		
	-- New number represents the adjusted probabilty of mission success	
	local iAdjustmentLimit = g_MissionList[iMissionNumber][ML_MISSION_SUCCESS] - iDefenderEspionagePosture + iAttackerEspionagePosture	-- Basic chance of success, - Defense IP + Attack IP
	if iAdjustmentLimit < g_iMinimumMissionAdjustment then iAdjustmentLimit = g_iMinimumMissionAdjustment end	-- ensure there is at least some chance of success
	
	if not pPlayer:IsHuman() then															-- Adds better chance for success for AI attacker, based on game difficulty  (sman:20180210)
		iAdjustmentLimit = iAdjustmentLimit + ( (g_iHandicap - 3) * 2 )						-- For Prince and below, buff; King +2% chance success, Emp +4, Immortal +6, Diety +8
	end
		
	local iMissionChance = math.random(1, 100)
	if iMissionChance < iAdjustmentLimit then											-- Was Covert Mission successful?  Based on adjusted mission probability for the basic mission (g_MissionList[iMissionNumber][ML_MISSION_SUCCESS])
		iMissionResult = YES
	end				
	--print("Mission result: " .. iMissionResult .. "    Normal success percent: " .. g_MissionList[iMissionNumber][ML_MISSION_SUCCESS] .. "   attacker EP: " .. iAttackerEspionagePosture .. "    defender EP: " .. iDefenderEspionagePosture .. "    Final Adjustment Limit: " .. iAdjustmentLimit .. "    Mission Chance: " .. iMissionChance)
	return iMissionResult
end


function GetMissionResult(tMissionResultData, iMissionNumber, pPlayer, sAgentName, iAgentRank, currentEra, pTargetPlayer, pTargetCity)	-- Function determines mission results
	--print("==> Inside GetMissionResult.  Mission: " .. g_MissionList[iMissionNumber][ML_MISSION_NAME] .. "    Mission Number: " .. iMissionNumber)
	local iOriginalMission = iMissionNumber													-- Save original mission in case of Mission Prevention
	local sTargetCityName = pTargetCity:GetName()											-- Saved name of the target city
	local NotificationType = -1																-- Set up defaults for the AddNotification function (nil)
	local NotificationUnit = nil															-- Most notification will display the Operative unit, but not always
	local NotificationStringFading = ""														-- Clear notification strings for later appending
	local NotificationStringLong = ""
	local NotificationX = pTargetCity:GetX()												-- Notification camera position will default to target city, not the attacking unit
	local NotificationY = pTargetCity:GetY()
	local MissionResult = NO																-- Ensure initial/default results are clear/negative
	local iAttackingOperatorLevel = iAgentRank or 1											-- Value for the level (1..3) of the operator launching an attack
	local sSpyRank = Locale.ConvertTextKey("TXT_KEY_SPY_RANK_" .. iAttackingOperatorLevel-1)	-- Text version of spy rank		
	local sReportTitle																		-- Strings for various report/notification popups				
	local sReconHeader
	local sReconReport
	local sReconSummary
	
	-- ==================================================================================	
	-- ======================		Mission Prevention		=========================
	-- If prevented, change iMissionNumber to 99 - "MISSION_PREVENTED" - trap at end of the next IF

	local iAdjustedEP = math.floor( CalculateEspionagePosture(pTargetPlayer, pTargetCity) / 2 ) -- What is half of the target city owner's EP?
	if iAdjustedEP > 25 then iAdjustedEP = 25 end											-- Max chance to prevent mission = 25%
	if math.random(1, 100) < iAdjustedEP then												-- If random # is < half the local owner's EP, then... 
		iMissionNumber = MISSION_PREVENTED													-- Mission was prevented before being launched!  Process like a normal mission below		
	end

	-- ==================================================================================	Individual Mission Effects		
	-- ======================		Mission: Conduct Recon		=========================
	if iMissionNumber == CONDUCT_RECON then													-- No tests for human player, as AI never calls this mission
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results; probably human only - AI shouldn't launch this mission				
		if MissionResult == YES then														-- Conduct Recon mission was successful			
			sReportTitle = "Reconnaisance Report"
			sReconHeader, sReconReport, sReconSummary = ProcessReconMissionResults(sAgentName, iAgentRank, pTargetCity)	-- Get the results of the recon mission - even if it's a failure						
			NotificationStringFading = "Successful reconnaisance near " .. sTargetCityName .. "!"  
			NotificationStringLong = sSpyRank .. " " .. sAgentName .. " conducted a successful reconnaissance mission near " .. sTargetCityName .. "![NEWLINE][NEWLINE]Report Summary: " .. sReconSummary
		else																				-- Recon mission failed; only notification actions are taken			
			NotificationStringFading = "Attempted reconnaissance near " .. sTargetCityName .. " failed."  
			NotificationStringLong = sSpyRank .. " " .. sAgentName .." failed in an attempt to conduct reconnaissance near " .. sTargetCityName .. "."
		end

	-- ======================		Mission: Poison Wells (changed from Poison Water, sman:20180209)		=========================
	elseif iMissionNumber == POISON_WATER then																
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results
		if MissionResult == YES then														-- Poison mission was successful
			local popLoss = math.random(1, currentEra + 2)									-- Get population to be lost, influenced by Attacker civ's Era (higher losses in later Eras)
			local targetCityPop = pTargetCity:GetPopulation()								-- Get current target city pop			
			if popLoss >= targetCityPop then popLoss = targetCityPop - 1 end				-- Prevent pop from falling below 1			
			local newPop = targetCityPop - popLoss											-- Determine new poplation for target city			
			pTargetCity:SetPopulation(newPop, true)											-- Set target city's pop to the new (lower) number
			pTargetCity:SetFood(0)															-- Remove all extra food from city (sman:20180215)
																							
			NotificationStringFading = "The wells near " .. sTargetCityName .. " were poisoned!"  			
			if not pPlayer:IsHuman() then 													-- Set the notification for a successful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have poisoned wells in " .. sTargetCityName .. "!"				
			else			
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " successfully poisoned the wells near " .. sTargetCityName .. "!"
			end

			if popLoss == 0 then
				NotificationStringLong = NotificationStringLong .. "  However, due to the city's small size, the overall population did not change."
			else			
				NotificationStringLong = NotificationStringLong .. "  The city lost " .. popLoss .. " population."
			end
		else																				-- Poison mission failed; only notification actions are taken
			NotificationStringFading = "An attempt to poison the wells near " .. sTargetCityName .. " failed."  
			if not pPlayer:IsHuman() then 													-- Set the notification for a successful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents failed in their attempt to poison wells in " .. sTargetCityName .. "!"				
			else
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to poison the wells near " .. sTargetCityName .. "."
			end
		end

	-- ======================		Mission: Spread Rumors and Propaganda		===========================
	elseif (iMissionNumber == SPREAD_RUMORS) or (iMissionNumber == SPREAD_PROPAGANDA) then	
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results
		if MissionResult == YES then														-- Spread Rumors/Propaganda mission was successful
			local missionType = "rumors"													-- Default mission is Rumors
			local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or "Enemy"		-- Target city owner civ adjective
			local lowerRange = 10															-- Range for Rumors is 10..25
			local higherRange = 25					
			if iMissionNumber == SPREAD_PROPAGANDA then										-- Range for Propaganda is 5..66
				missionType = "propaganda"
				lowerRange = 5
				higherRange = 66
			end
			
			local adjustmentCoefficient = math.random(lowerRange, higherRange)				-- Determine where in the range we set the random adjustment
			local oldCulture = pTargetPlayer:GetJONSCulture()								-- Get the existing Culture amount
			local newCulture = math.ceil( oldCulture * (adjustmentCoefficient/100) )		-- Determine new Culture amounts as the old amount * (adjustment / 100)
			if newCulture < 1 then newCulture = 1 end										-- Ensure Culture is always a positive, non-zero number
			local iCultureDelta = oldCulture - newCulture
			if iCultureDelta < 1 then iCultureDelta = 1 end
			pTargetPlayer:SetJONSCulture(newCulture)										-- Set the new, adjusted culture amount
				
			adjustmentCoefficient = math.random(lowerRange, higherRange)					-- Determine where in the range we set the random adjustment
			local oldGAPoints = pTargetPlayer:GetGoldenAgeProgressMeter()					-- Get the existing Culture amount
			local newGAPoints = math.ceil( oldGAPoints * (adjustmentCoefficient/100) )		-- Determine new Culture amounts as the old amount * (adjustment / 100)
			if newGAPoints < 1 then newGAPoints = 1 end										-- Ensure Culture is always a positive, non-zero number
			local iAGDelta = oldGAPoints - newGAPoints 
			if iAGDelta < 1 then iAGDelta = 1 end
			pTargetPlayer:SetGoldenAgeProgressMeter(newGAPoints)							-- Set the new, adjusted culture amount
			NotificationStringFading = "Spreading of " .. missionType ..  " reduces " .. targetAdjective .. " progress!"

			if not pPlayer:IsHuman() then 													-- Set the notification for a successful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents spreading " .. missionType .. " in " .. sTargetCityName .. " have reduced " .. targetAdjective .. " [ICON_CULTURE] Culture progress by " .. iCultureDelta .. " and [ICON_GOLDEN_AGE] Golden Age progress by " .. iAGDelta .."!"				
			else	
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " successfully spread " .. missionType .. " in " .. sTargetCityName .. ". This has reduced " .. targetAdjective .. " [ICON_CULTURE] Culture progress by " .. iCultureDelta .. " and [ICON_GOLDEN_AGE] Golden Age progress by " .. iAGDelta .."!"				
			end
		else																				-- Poison mission failed; only notification actions are taken
			local missionType = "rumors"
			if iMissionNumber == SPREAD_PROPAGANDA then missionType = "propaganda" end
			NotificationStringFading = "An attempt to spread " .. missionType .. " near " .. sTargetCityName .. " failed."  

			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents failed in their attempts to spread " .. missionType .. " in " .. sTargetCityName .. "."				
			else																			-- Set notification for unsuccessful human player attack
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to spread " .. missionType .. " in " .. sTargetCityName .. "."		
			end
		end

	-- ======================		Mission: Sabotage Unit		===========================
	elseif iMissionNumber == SABOTAGE_UNIT then			
		local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or ""				-- Target city owner civ adjective
		local iLuckyUnitID = GetRandomUnit(pTargetCity)										-- Function returns unit ID of an enemy unit near the target city; -1 if none found
		local pLuckyUnit = pTargetPlayer:GetUnitByID(iLuckyUnitID)							-- Get lucky unit pointer
		local sLuckyUnitName = ""
		if not pLuckyUnit then																-- Set mission result to "no GP found" if can't build target unit pointer
			iLuckyUnitID = -1																-- Value means no unit found
			sLuckyUnitName = "unit"															-- Generic "unit" used for a unit name
		else																				-- If pLuckyUnit isn't nil, then...
			sLuckyUnitName = pLuckyUnit:GetName() 											-- Get unit's name 
		end										
		
		--print("Unit name: " .. sLuckyUnitName .. "    ID: " .. iLuckyUnitID)
				
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results				
		
		if (MissionResult == YES) and (iLuckyUnitID > -1) then								-- Sabotage unit succeeded and there are units near the city
			tMissionResultData[8] = pLuckyUnit:GetUnitType()								-- Save pLuckyUnit type
			NotificationX = pLuckyUnit:GetX()												-- Change notification lookat to unit location, not target city
			NotificationY = pLuckyUnit:GetY()
			
			local sDamageReport = ""
			local iUnitCurrentDamage = pLuckyUnit:GetDamage()								-- Get current unit damage
			local iUnitNewDamage = math.random( (iAttackingOperatorLevel * 5 ), 75) + ( iAttackingOperatorLevel * 20 )	-- Damage range: L1 = (5..95), L3 = (15, 135)
			if (iUnitCurrentDamage + iUnitNewDamage) > 100 then								-- If current + new damage > 100, remove the unit
				iUnitDamage = 100 
				sDamageReport = " The sabotage was so severe it destroyed the unit!"				
				NotificationType = 2														-- "Unit Killed" notification	
				pLuckyUnit:Kill()															-- Remove the unit from the game
			else
				sDamageReport = " The sabotage resulted in " .. iUnitNewDamage .. " HP damage to the unit!"				
				pLuckyUnit:SetDamage(iUnitCurrentDamage + iUnitNewDamage)
			end

			NotificationStringFading = targetAdjective .. " " .. sLuckyUnitName .. " sabotaged near  " .. sTargetCityName .. "!" 			
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents sabotaged a " .. targetAdjective .. " " .. sLuckyUnitName .. " near the city of " .. sTargetCityName .. "!"
			else		
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " has sabotaged a " .. targetAdjective .. " " .. sLuckyUnitName .. " near the city of " .. sTargetCityName .. "!"
			end	
			NotificationStringLong = NotificationStringLong .. sDamageReport
			
		else		
			NotificationStringFading = "Attempt to sabotage a " .. sLuckyUnitName .. " near " .. sTargetCityName .. " failed."  
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have failed in their attempt to sabotage a " .. sLuckyUnitName .. " near the " .. targetAdjective .. " city of " .. sTargetCityName .. "."
			else
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to sabotage a " .. sLuckyUnitName .. " near the " .. targetAdjective .. " city of " .. sTargetCityName .. "."				
			end
		end
	
	-- ======================		Mission: Pillage Improvement		===================
	elseif iMissionNumber == PILLAGE_IMPROVEMENT then			
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results
		local pPlot = GetRandomImprovement(pTargetCity)										-- Update the plot pointer - get a random plot with an improvement or nil
		local improvementName = "Improvement"
		if not pPlot then																	-- No pPlot probably means no improvements in city; automatic fail
			MissionResult = NO 
		else
			improvementName = GetImprovementString(pPlot)									-- Get the name of the improvement on the plot
			if (not improvementName) or (improvementName == "") then improvementName = "improvement"	end		-- If no good name, give generic name
		end								
		
		if MissionResult == YES then														-- Pillage mission succeeded	
			pPlot:SetImprovementPillaged(true)												-- Set the plot status to pillaged
			NotificationX = pPlot:GetX()													-- Notification camera position will default to target city, not the attacking unit
			NotificationY = pPlot:GetY()
			NotificationStringFading = "Terrain improvement pillaged!"						-- File the Notification fade message
			if not pPlayer:IsHuman() then 													-- Set the notification for a successful AI attack; change Notification type from human attacks
				NotificationType = 5														-- This is the Red Spy icon				
				NotificationStringLong = "Foreign agents pillaged a " .. improvementName .. " near " .. sTargetCityName .. "!"  
			else				
				NotificationType = 4														-- This is the Blue Spy icon		
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " pillaged a " .. improvementName .. " near " .. sTargetCityName .. "!" 			
			end						
		else																				-- Poison mission failed; only notification actions are taken
			if not pPlayer:IsHuman() then								 					-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationType = 4														-- This is the Blue Spy icon	
			else
				NotificationType = 5														-- This is the Red Spy icon				
			end
			NotificationStringFading = "Attempt to pillage " .. improvementName .. " near " .. sTargetCityName .. " failed."  
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents failed in their attempts to pillage terrain improvements near " .. sTargetCityName .. "."				
			else		
				if pPlot then 
					NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed an attempt to pillage a(n) " .. improvementName .. " near " .. sTargetCityName .. "." 		
				else
					NotificationStringLong = sSpyRank .. " " .. sAgentName .. " found no improvements to pillage near " .. sTargetCityName .. "." 		
				end
			end
		end

	-- ======================		Mission: Assassinate Great Person		===================
	elseif iMissionNumber == ASSASSINATE_GREAT_PERSON then	
		local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or ""				-- Target city owner civ adjective
		local iLuckyGreatPersonID = GetRandomGreatPerson(pTargetCity)						-- Function returns unit ID of an enemy great person near the target city; -1 if none found
		local pGreatPerson = pTargetPlayer:GetUnitByID(iLuckyGreatPersonID)					-- Get Great Person unit pointer
		if not pGreatPerson then iLuckyGreatPersonID = -1 end								-- Set mission result to "no GP found" if can't build target unit pointer
		
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results				
		if (MissionResult == YES) and (iLuckyGreatPersonID > -1) then						-- Assassinate GP mission succeeded	and there are GP's near the city
			
			tMissionResultData[8] = pGreatPerson:GetUnitType()								-- Save pGreatPerson type
			NotificationX = pGreatPerson:GetX()												-- Change notification lookat to unit location, not target city
			NotificationY = pGreatPerson:GetY()
			
			sGreatPersonName = pGreatPerson:GetName() or ", name unknown,"					-- Get GP's name with (type)				
			NotificationStringFading = targetAdjective .. " Great Person assassinated in  " .. sTargetCityName .. "!" 			
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents assassinated " .. targetAdjective .. " Great Person " .. sGreatPersonName .. " in the city of " .. sTargetCityName .. "!"
			else		
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " has assassinated " .. targetAdjective .. " Great Person " .. sGreatPersonName .. " in the city of " .. sTargetCityName .. "!"
			end			
			NotificationType = 2															-- "Unit Killed" notification		
			pGreatPerson:Kill()																-- Remove the unit from the game
		else		
			NotificationStringFading = "Attempt to assasinate a Great Person in " .. sTargetCityName .. " failed."  
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have failed in their attempts to assassinate a Great Person in the " .. targetAdjective .. " city of " .. sTargetCityName .. "."
			else
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to assassinate a Great Person in the " .. targetAdjective .. " city of " .. sTargetCityName .. "."
				if iLuckyGreatPersonID < 0 then
					NotificationStringLong = NotificationStringLong .. " None could be found."
				end
			end
		end
		
	-- ======================		Mission: Sabotage Production		===================
	elseif iMissionNumber == SABOTAGE_PRODUCTION then	
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results
		if MissionResult == YES then														-- Sabotage Production mission succeeded				
			local buildingProduced =pTargetCity:GetProductionBuilding() or -1				-- Returns the building type ID
			local projectProduced = pTargetCity:GetProductionProject() or -1				-- Returns?
			local unitProduced = pTargetCity:GetProductionUnit() or -1						-- Returns the unit ID
			
			if buildingProduced ~= -1 then													-- City is producing a building; buildingProduced = building type
				local hammersInvested = pTargetCity:GetProduction()							-- How much production already completed?
				local newHammers															-- How many hammers to remove?			
				local buildingType = Locale.ConvertTextKey(GameInfo.Buildings[buildingProduced].Description) or "Building"		-- Building description

				local iMaxGlobInst = GameInfo.BuildingClasses[GameInfo.Buildings[buildingProduced].BuildingClass].MaxGlobalInstances or 0	-- Change: attempt to resolve too many nil value errors in testing (sman:20180215)
				local iMaxPlayInst = GameInfo.BuildingClasses[GameInfo.Buildings[buildingProduced].BuildingClass].MaxPlayerInstances or 0	-- Change: attempt to resolve too many nil value errors in testing (sman:20180215)

				--print("Building produced: " .. buildingType .. "    ID: " .. buildingProduced .. "    iMaxGlobInst: " .. iMaxGlobInst .. "    iMaxPlayInst: " .. iMaxPlayInst)

				if iMaxGlobInst == 1 then													-- World Wonder
					newHammers = math.ceil( hammersInvested * (math.random(10, 33) / 100) )	-- How many hammers to remove?
				elseif iMaxPlayInst == 1 then												-- National Wonder
					newHammers = math.ceil( hammersInvested * (math.random(25, 50) / 100) )	-- How many hammers to remove?
				else																		-- Normal building
					newHammers = math.ceil( hammersInvested * (math.random(50, 90) / 100) )	-- How many hammers to remove?
				end
				pTargetCity:ChangeProduction(-newHammers)									-- Remove the hammers 

				NotificationStringFading = buildingType .. " production sabotaged in " .. sTargetCityName .. "."
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " sabotaged " .. buildingType .. " production, destroying " .. newHammers .. " hammers of progress in " .. sTargetCityName .. "."

			elseif projectProduced ~= -1 then												-- City is producing a project; projectProduced = project type
				local hammersInvested = pTargetCity:GetProduction()							-- How much production already completed?
				local newHammers = math.ceil( hammersInvested * (math.random(25, 50) / 100) )	-- How many hammers to remove?
				local projectType = Locale.ConvertTextKey(GameInfo.Projects[projectProduced].Description) or "Project"
				
				pTargetCity:ChangeProduction(-newHammers)									-- Remove the hammers 

				NotificationStringFading = projectType .. " production sabotaged in " .. sTargetCityName .. "."
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " sabotaged " .. projectType .. " production, destroying " .. newHammers .. " hammers of progress in " .. sTargetCityName .. "."

			elseif unitProduced ~= -1 then													-- City is producing a unit; unitProduced = unit type
				local unitType = Locale.ConvertTextKey(GameInfo.Units[unitProduced].Description) or "Unit"
				
				pTargetCity:SetProduction(1)												-- Set production to 1
				NotificationStringFading = unitType .. " production sabotaged in " .. sTargetCityName .. "."
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " sabotaged " .. unitType .. " production in " .. sTargetCityName .. "."
			else																			-- City producing something that cannot be sabotaged
				NotificationStringFading = "There is nothing that can be sabotaged in " .. sTargetCityName .. "."
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " can find nothing worth sabotaging in " .. sTargetCityName .. "."
				tMissionResultData[2] = NotificationStringFading							-- If no valid target, exit early; different result from failing to sabotage...
				tMissionResultData[3] = NotificationStringLong								-- Populate results table
				tMissionResultData[6] = NotificationX
				tMissionResultData[7] = NotificationY
				tMissionResultData[8] = MissionResult
				return																		-- Exit early
			end							
		else																				-- Poison mission failed; only notification actions are taken						
			NotificationStringFading = "Attempt to sabotage production in " .. sTargetCityName .. " has failed." 
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents failed in their attempts to sabotage production in " .. sTargetCityName .. "."				
			else		
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to sabotage production in " .. sTargetCityName .. "." 		
			end
		end

	-- ======================		Mission: Steal Gold		===================
	elseif iMissionNumber == STEAL_GOLD then
		local upperRange = 10																-- Default max gold to steal is 10 (or 10%)
		if pTargetCity:IsCapital() then														-- City is capital, so higher amount of gold is possible to steal
			upperRange = 25																	-- For the captial the upper max to steal is 25% of all a civ's gold
		end
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results
		if MissionResult == YES then														-- Steal Gold mission succeeded			
			local targetGold = pTargetPlayer:GetGold()										-- Get total gold target civ has on hand
			local stolenAmount = math.ceil( targetGold * (math.random(1, upperRange) / 100) )	-- Determines how much gold was stolen, based on if city is capital or not
			targetGold = targetGold - stolenAmount											-- Determine new gold amount
			if stolenAmount < 1 then stolenAmount = 1 end									-- Ensure it's now lower than 1 gold piece
			pTargetPlayer:SetGold(targetGold)												-- Set gold amount at new level
			pPlayer:ChangeGold(stolenAmount)												-- Adds the stolen gold amount to your treasury

			NotificationStringFading = stolenAmount .. " [ICON_GOLD] Gold stolen from " .. sTargetCityName .. "."
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents stole " .. stolenAmount .. " [ICON_GOLD] Gold from " .. sTargetCityName .. "."
			else
				NotificationStringLong = sSpyRank .. " " .. sAgentName .." stole " .. stolenAmount .. " [ICON_GOLD] Gold from " .. sTargetCityName .. "."
			end
		else			
			NotificationStringFading = "Attempt to steal gold from " .. sTargetCityName .. " has failed."  
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have failed in their attempts to steal gold from " .. sTargetCityName .. "." 	
			else
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to steal gold from " .. sTargetCityName .. "." 	
			end
		end

	-- ======================		Mission: Spread Plague		===================
	elseif iMissionNumber == SPREAD_PLAGUE then			
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results
		if MissionResult == YES then														-- Spread Plague mission succeeded			
			local popLoss = math.random(1, currentEra + 2)									-- Get population to be lost, influenced by attacker's Era (higher losses in later Eras)
			local targetCityPop = math.floor(pTargetCity:GetPopulation())					-- Get current target city pop			
			local tradePartnerImpact = NO													-- Tracks if any trade partner cities were hit with plague as well
			if popLoss >= targetCityPop then popLoss = targetCityPop - 1 end				-- Prevent pop from falling below 1			
			local newPop = targetCityPop - popLoss											-- Determine new poplation for target city			
			pTargetCity:SetPopulation(newPop, true)											-- Set target city's pop to the new (lower) number			
			pTargetCity:SetFood(0)															-- Remove all extra food from city (sman:20180215)

			local targetTrade = pTargetPlayer:GetTradeRoutes()								-- Check if trade partners hit with plague
			for i,v in ipairs(targetTrade) do												-- Scroll through all trade routes for the target player
				if v.FromCity:GetID() == pTargetCity:GetID() then							-- if the trade route originates in target city, then...
					if math.random(1, 100) < 50 then										-- 50:50 chance the city will be hit or not
						if v.ToCity:GetPopulation() >= 2 then								-- if impacted city pop is 2 or more, reduce population, set impact flag to yes
							v.ToCity:ChangePopulation(-1, true)								-- Actually reduce the pop
							tradePartnerImpact = YES										-- Set impact flag on
						end
					end					
				end
				if v.ToCity:GetID() == pTargetCity:GetID() then								-- if the trade route terminates in target city, then...
					if math.random(1, 100) < 50 then										-- 50:50 chance the city will be hit or not
						if v.FromCity:GetPopulation() >= 2 then								-- if impacted city pop is 2 or more, reduce population, set impact flag to yes
							v.FromCity:ChangePopulation(-1, true)							-- Actually reduce the pop
							tradePartnerImpact = YES										-- Set impact flag on
						end
					end					
				end
			end
						
			NotificationStringFading = "Plague has struck the city of " .. sTargetCityName .. "!"  
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have successfully spread plague throughout the city of " .. sTargetCityName .. "." 	
			else
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " has successfully caused a plague epidemic in the city of " .. sTargetCityName .. "!"
			end

			if popLoss == 0 then
				NotificationStringLong = NotificationStringLong .. "  However, due to the city's small size, the overall population did not change."
			else			
				NotificationStringLong = NotificationStringLong .. "  The city lost " .. popLoss .. " population."
			end

			if tradePartnerImpact == YES then
				NotificationStringLong = NotificationStringLong .. "  The contagion has spread to trading partners of this city!"
			end
		else																				-- Poison mission failed; only notification actions are taken
			NotificationStringFading = "An attempt to spread plague in the city of " .. sTargetCityName .. " failed."  
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have failed in their attempts to spread plague in the city of " .. sTargetCityName .. "." 	
			else
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to spread plague to " .. sTargetCityName .. "."
			end
		end

	-- ======================		Mission: Destroy (normal) Building		===================
	elseif iMissionNumber == DESTROY_BUILDING then	
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results
		local destroyedBuildingType	= -1													-- Will hold the random normal building type in a city to by destroyed

		if MissionResult == YES then														-- Destroy Building mission succeeded				
			destroyedBuildingType = GetRandomBuilding(pTargetCity)							-- Returns a random building in city instance sent as argument; non-national and non-world wonder buildings returned	
						
			if destroyedBuildingType ~= -1 then												-- If ~= -1 then there was a valid normal building found
				local buildingName = Locale.ConvertTextKey(GameInfo.Buildings[destroyedBuildingType].Description) or "Building"		-- Building description

				pTargetCity:SetNumRealBuilding(destroyedBuildingType, 0)					-- Removes building type from city			
				NotificationStringFading = buildingName .. " destroyed in the city of " .. sTargetCityName .. "!"  

				if not pPlayer:IsHuman() then 												-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
					NotificationStringLong = "Foreign agents destroyed a " .. buildingName .. " in the city of " .. sTargetCityName .. "!"
				else
					NotificationStringLong = sSpyRank .. " " .. sAgentName .. " destroyed a " .. buildingName .. " in the city of " .. sTargetCityName .. "!"
				end
			end
		end
		
		if MissionResult ~= YES	or destroyedBuildingType == -1 then							-- Destroy Building failed or no normal buildings to destroy; only notification actions are taken			
			MissionResult = NO																-- No buildings to destroy, so change result to failure
			NotificationStringFading = "Attempt to destroy a building in " .. sTargetCityName .. " failed."  
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have failed in their attempts to destroy a building in the city of " .. sTargetCityName .. "."	
			else
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to destroy a building in the city of " .. sTargetCityName .. "."
			end
		end

	-- ======================		Mission: Recruit Partisans		===================
	elseif iMissionNumber == RECRUIT_PARTISANS then
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results
		local numUnitsSpawned = -1															-- Defaults to a nil/negative result
		local spawnUnitType = -1															-- Defaults to a nil/negative result
	
		if MissionResult == YES then														-- Recruit Partisans mission succeeded	
			local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or ""			-- Target city owner civ adjective
									
			numUnitsSpawned, spawnUnitType = SpawnRebelUnits(pTargetCity, BARBARIAN_PLAYER)  -- Function generates number of units based on CurrentEra-3 near target city; arg 3 is playerID of units to spawn
			if numUnitsSpawned > 0 then
				local unitName = Locale.ConvertTextKey(GameInfo.Units[spawnUnitType].Description)	-- Get the spawned unit's description (name)
				local unitDescriptor = "unit"												-- Is it a 'unit' or 'units' depending on how many spawned - makes the grammar better in the Notification
				if numUnitsSpawned > 1 then unitDescriptor = "units" end

				NotificationType = 3														-- "Rebels Apprear" notification
				tMissionResultData[8] = spawnUnitType										-- unitID type
				NotificationStringFading = targetAdjective .. " partisans have rebelled near " .. sTargetCityName .. "!"  

				if not pPlayer:IsHuman() then 												-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
					NotificationStringLong = "Foreign agents have raised an army of " .. targetAdjective .. " partisans, consisting of " .. numUnitsSpawned .. " " .. unitName .. " " .. unitDescriptor .. ", near the city of " .. sTargetCityName .. "!"
				else
					NotificationStringLong = sSpyRank .. " " .. sAgentName .. " raised an army of " .. targetAdjective .. " partisans, consisting of " .. numUnitsSpawned .. " " .. unitName .. " " .. unitDescriptor .. ", near the city of " .. sTargetCityName .. "!"
				end
			end
		end
		
		if MissionResult ~= YES	or numUnitsSpawned < 1 then
			NotificationStringFading = "Attempt to raise rebellion in " .. sTargetCityName .. " failed."  
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have failed in their attempts to raise rebellion in the city of " .. sTargetCityName .. "."	
			else
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to raise rebellion in the city of " .. sTargetCityName .. "."
			end
		end

	-- ======================		Mission: Cause Resistance		===================
	elseif iMissionNumber == CAUSE_RESISTANCE then
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results
		local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or ""				-- Target city owner civ adjective
		if MissionResult == YES then														-- Cause Resistance mission succeeded				
			local resistanceDuration = math.random(3, 9)									-- Determine the number of turns the city will enter resistance (adding 1 since city loses 1 resist when turn starts - since AI attackes between turns, this is always 1 turn off)
			local taretCityPop = pTargetCity:GetPopulation()
			if resistanceDuration > taretCityPop then resistanceDuration = taretCityPop end			
			pTargetCity:ChangeResistanceTurns(resistanceDuration)			
						
			NotificationStringFading = "The city of " .. sTargetCityName .. " is experiencing resistance!" 
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have created resistance in the " .. targetAdjective .. " city of " .. sTargetCityName .. "!"
			else		
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " has incited resistance in the " .. targetAdjective .. " city of " .. sTargetCityName .. "!"
			end
		else		
			NotificationStringFading = "Attempt to create resistance in " .. sTargetCityName .. " failed."  
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have failed in their attempts to create resistance in the " .. targetAdjective .. " city of " .. sTargetCityName .. "."
			else
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to incite resistance in the " .. targetAdjective .. " city of " .. sTargetCityName .. "."
			end
		end

	-- ======================		Mission: Cause Anarchy		===================
	elseif iMissionNumber == CAUSE_ANARCHY then
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results
		local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or ""				-- Target city owner civ adjective			
		if MissionResult == YES then														-- Cause Anarchy mission succeeded				
			local anarchyDuration = math.random(1, 3)										-- Determine the number of turns the target civ will enter anarchy (no gold income, science, or production)
			if pTargetPlayer:IsAnarchy() then												-- If civ is already in anarchy, this will add anarchyDuration turns
				pTargetPlayer:ChangeAnarchyNumTurns(anarchyDuration)
			else																			-- If not already in anarchy, will set anarchy on for anarchyDuration turns
				pTargetPlayer:SetAnarchyNumTurns(anarchyDuration)
			end			
						
			NotificationStringFading = "The " .. targetAdjective .. " civilization is experiencing anarchy!"  
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have created anarchy throughout the " .. targetAdjective .. " civilization!"
			else	
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " has incited anarchy throughout the " .. targetAdjective .. " civilization!"
			end
		else					
			NotificationStringFading = "Attempt to create anarchy in the " .. targetAdjective .. " civilization failed."  			
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have failed in their attempts to create anarchy in the " .. targetAdjective .. " civilization."
			else			
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to incite anarchy in the " .. targetAdjective .. " civilization."
			end
		end



-- ======================		Mission: Nuclear Suitcase		===================
	elseif iMissionNumber == NUKE_SUITCASE then
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results
		local attackerAdjective = pPlayer:GetCivilizationAdjective() or ""					-- Attacking civ adjective
		local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or ""				-- Target city owner civ adjective
		if MissionResult == YES then														-- Nuke Suitcase mission succeeded				
			local iLossPercent = 25															-- Manage population loss from attack
			local bIsFalloutShelter = false
			if pTargetCity:GetNumBuilding(BUILDING_BOMB_SHELTER) > 0 then					-- Does this city have this building?
				--print("City " .. pTargetCity:GetName() .. " has a bomb shelter")			-- If so, lower damage stats
				iLossPercent = 10
				bIsFalloutShelter = true			
			end
			
			local iCityDamage = math.random(50, 125)										-- Set city damage
			if bIsFalloutShelter then iCityDamage = math.random(25, 75) end
			pTargetCity:ChangeDamage(iCityDamage)

			local iTargetCityPop = pTargetCity:GetPopulation()								-- Get current target city pop
			local iMaxPopLoss = math.floor(iTargetCityPop * (iLossPercent/100))	+ 1		
			local iPopLoss = math.random(1, iMaxPopLoss)									-- Get population to be lost, influenced by Attacker civ's Era (higher losses in later Eras)
			pTargetCity:SetPopulation(iTargetCityPop - iPopLoss, true)						-- Set target city's pop to the new (lower) number
			
			local iNumTilesDamaged, iNumUnitsKilled = SetNukeSuitcaseDamage(pTargetCity, bIsFalloutShelter)	-- Function returns # of tiles with fallout, can be 0
						
			NotificationStringFading = "The city " .. sTargetCityName .. " has been attacked with a Nuclear Suitcase!"  

			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have detonated a Nuclear Suitcase in the " .. targetAdjective .. " city of " .. sTargetCityName .. "!" 
			else	
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " has detonated a Nuclear Suitcase in the " .. targetAdjective .. " city of " .. sTargetCityName .. "!" 
			end

			if iPopLoss > 0 then
				NotificationStringLong = NotificationStringLong .. " The city has lost " .. iPopLoss .. " population!"
			end
			if iNumUnitsKilled > 0 then
				NotificationStringLong = NotificationStringLong .. " Explosive damage has destroyed " .. iNumUnitsKilled .. " total units!"
			end			
			if iNumTilesDamaged >0 then
				NotificationStringLong = NotificationStringLong .. " Fallout has spread to " .. iNumTilesDamaged .. " tiles near the city!"
			end

		else		
			NotificationStringFading = "Attempt to detonate a Nuclear Suitcase in the city of " .. sTargetCityName .. " failed."  
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have failed in their attempts to detonate a Nuclear Suitcase in the city of " .. sTargetCityName .. "."
			else			
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to detonate a Nuclear Suitcase in the city of " .. sTargetCityName .. "."
			end
		end
			
	-- ======================		Mission: Steal City		===================
	elseif iMissionNumber == STEAL_CITY then
		MissionResult = GetMissionOutcome(iMissionNumber, pPlayer, pTargetCity)				-- Get mission results
		local attackerAdjective = pPlayer:GetCivilizationAdjective() or ""					-- Attacking civ adjective
		local targetAdjective = pTargetPlayer:GetCivilizationAdjective() or ""				-- Target city owner civ adjective
		if MissionResult == YES then														-- Steal City mission succeeded				
			local numUnitsSpawned, spawnUnitType = SpawnRebelUnits(pTargetCity, pTargetPlayer:GetID())  -- Function generates number of units based on target civ's CurrentEra - 3 near target city						
			NotificationStringFading = "The " .. targetAdjective .. " city " .. sTargetCityName .. " has joined the " .. attackerAdjective .. " empire!"  

			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have inspired the citizens of the " .. targetAdjective .. " civilization's city of " .. sTargetCityName .. " to join the " .. attackerAdjective .. " empire!" 
			else	
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " has inspired the citizens of the " .. targetAdjective .. " city of " .. sTargetCityName .. " to rebel, and join the " .. attackerAdjective .. " empire!" 
			end

			if numUnitsSpawned > 0 then
				local unitName = Locale.ConvertTextKey(GameInfo.Units[spawnUnitType].Description)	-- Get the spawned unit's description (name)
				NotificationStringLong = NotificationStringLong .. "  Loyalist " .. unitName .. " units have risen up near the city!"
			end

			pPlayer:AcquireCity(pTargetCity, true, false)									-- pPlayer acquires target city, 2nd arg=Conquest, 3rd arg=Trade); move to hear, as after transfer pTargetCity instance disappears
		else		
			NotificationStringFading = "Attempt to steal the city of " .. sTargetCityName .. " failed."  
			if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
				NotificationStringLong = "Foreign agents have failed in their attempts to inspire the city of " .. sTargetCityName .. " to leave the " .. targetAdjective .. " empire."
			else			
				NotificationStringLong = sSpyRank .. " " .. sAgentName .. " failed in an attempt to inspire the city of " .. sTargetCityName .. " to leave the " .. targetAdjective .. " empire."
			end
		end
	-- ======================		Mission: Abort Mission		===================
	elseif iMissionNumber == ABORT_MISSION then												-- Abort Mission - always available selection		
		return																				-- Exit early - no notifications since nothing happened...
	
		
	-- ======================		Mission: Mission Prevented		===================
	elseif iMissionNumber == MISSION_PREVENTED then											-- Mission Prevented - when iMissionNumber is changed above
		NotificationStringFading = "A planned Covert Op in the city of " .. sTargetCityName .. " has been prevented."  
		
		if not pPlayer:IsHuman() then 													-- Set the notification for an unsuccessful AI attack; change Notification type from human attacks
			NotificationStringLong = "Local security forces have prevented a `" .. g_MissionList[iOriginalMission][ML_MISSION_NAME] .. "` Covert Op in the city of " .. sTargetCityName .. " before it could be launched."
		else			
			NotificationStringLong = "The `" .. g_MissionList[iOriginalMission][ML_MISSION_NAME] .. "` Covert Op that " .. sSpyRank .. " " .. sAgentName .. " had planned in the city of " .. sTargetCityName .. " was crushed by local authorities while still in the planning stage. The co-conspirators have been scattered."
		end		
		MissionResult = PREVENTED	
	

	else
		print("Big error here ----  couldn't find a valid mission....")
		return
	end
	
	-- Set up the correct Notification type (either 4 or 5), unless a function above changed it.  NOTE: if both attacker & target are AI, mod usually won't display a notification			
	if NotificationType == -1 then																-- Notification type was not changed above, so change defaults
		if MissionResult == YES	then															-- Mission was successful, so...
			NotificationType = 4																-- Use the Blue Spy icon (we're assuming the human player is attacker)
			if not pPlayer:IsHuman() then								 						-- If the attacker is AI, then...    (pPlayer=attacker)
				NotificationType = 5															-- Use the Red Spy icon	(human player is target)			
			end
		else																					-- If mission failed, then....
			NotificationType = 5																-- Use the Red Spy icon	(human player is attacker)			
			if not pPlayer:IsHuman() then								 						-- If the attacker is AI, then...
				NotificationType = 4															-- Use the Blue Spy icon (human player is target)			
			end
		end	
	end
	tMissionResultData[1] = NotificationType												-- Built return table data
	tMissionResultData[2] = NotificationStringFading
	tMissionResultData[3] = NotificationStringLong	
	tMissionResultData[5] = MissionResult
	tMissionResultData[7] = sReconReport
	tMissionResultData[9] = NotificationX
	tMissionResultData[10] = NotificationY

	-- ====================================================================================
	--		Collect mission stats for later analysis
	g_MissionList[iOriginalMission][ML_NUM_ATTEMPTS] = g_MissionList[iOriginalMission][ML_NUM_ATTEMPTS] + 1
	
	if MissionResult == YES then
		g_MissionList[iOriginalMission][ML_NUM_SUCCESS] = g_MissionList[iOriginalMission][ML_NUM_SUCCESS] + 1
	elseif MissionResult == PREVENTED then
		g_MissionList[iOriginalMission][ML_NUM_PREVENTS] = g_MissionList[iOriginalMission][ML_NUM_PREVENTS] + 1
	end	

end

-- ========================================================================================
--	Process Human Player Covert Missions:  Only arrive here by way of DisplayCOMenuPopup() in 10_EO_Master_Process.lua
-- ========================================================================================

function ProcessMission (iMissionNumber, g_pSelectedCity)									-- Function determines mission results - FOR THE HUMAN PLAYER!  
	--print("==> Inside ProcessMission.  Mission: " .. g_MissionList[iMissionNumber][ML_MISSION_NAME] .. "    Mission Number: " .. iMissionNumber)
	if iMissionNumber == ABORT_MISSION then return end										-- User selected "Abort Mission" - no need for further processing

	local pPlayer = Players[Game.GetActivePlayer()]											-- Get basic player, unit, plot data for processing; pPlayer is the "attacking" player
	local currentEra = pPlayer:GetCurrentEra()												-- Get Active (human) player's current level
		
	local pTargetCity = g_pSelectedCity														-- Get target city, based on location of attacking spy
	local sTargetCityName = pTargetCity:GetName()											-- Saved name of the target city
	local pTargetPlayer = Players[pTargetCity:GetOwner()]									-- Get current player
	local sTargetPlayerAdjective = pTargetPlayer:GetCivilizationAdjective()					-- Target player's short description, i.e. "China"
	local iTargetCityX = pTargetCity:GetX() or -1											-- Get X location for target city
	local iTargetCityY = pTargetCity:GetY() or -1											-- Get Y location for target city
	local iMissionCost = GetMissionCost(pPlayer, iMissionNumber, pTargetPlayer, pTargetCity)	-- Get exact mission cost now, in case of "steal city" 		


	local bSpyInCity, sSpyName, iSpyRank, sSpyMission, iSpyPercentComplete, iSpyTurnsLeft = SpyInCityCheck(pPlayer, pTargetCity)	-- Check to see if pPlayer has a spy in pTargetCity
	if not bSpyInCity then																	-- Should never reach here - have to have a spy to get to the popup menu, which calls this function
		sSpyName = "X"																		-- Give generic value, i.e. "Special Agent X" 
		iSpyRank = 1																		-- Give lowes rank = 1
	end

	local sSeparatorString = "[NEWLINE][NEWLINE]"											-- Default the separator between messages in Notification with newlines
	--local sSeparatorString = "  "															-- Default the separator between messages in Notification with spaces
	local iTopIcon = 1																		-- Gold icon, blue bg
	local sLineSeperator = "------------------------------------------------------------------------------"	-- Various settings to formate the report
	
	--print("SPY RANK INDEX: " .. g_SpyRankIndexValue[iAgentRank])	
	--print("Agent rank: " .. g_SpyRankIndexValue[iAgentRank] .. "   name: " .. sAgentName .. "    target: " .. sTargetCityName .. "    mission: " .. iMissionNumber)
	
					
	-- Mission results:		    Notify Type		FadeStr		Mission Result	Reaction	Msn	Success		Incident	Summary		UnitSpawnType,	Notify X,		Notify Y-- default values for mission results data table
	--							1				2			3				4			5				6			7			8				9				10
	local tMissionResultData = { 4,				"",			"",				"",			1,				1,			"",			-1,				iTargetCityX,	iTargetCityY }
	
	GetMissionResult(tMissionResultData, iMissionNumber, pPlayer, sSpyName, iSpyRank, currentEra, pTargetPlayer, pTargetCity)	-- Get the results of this mission		
	
	GetInternationalResponses(tMissionResultData, iMissionNumber, pPlayer, pTargetPlayer, pTargetCity, sSpyName, iSpyRank)		-- Get the international response
	
	
	local sLongMessage = tMissionResultData[3]												-- Add the "Mission Result" string to the Notification long message
	if tMissionResultData[4] ~= "" then sLongMessage = sLongMessage .. sSeparatorString .. tMissionResultData[4] end	-- Add the "Incident result" string to the Notification long message, if there is one
		
	SendNotification(tMissionResultData[1], sLongMessage, tMissionResultData[2], tMissionResultData[8], tMissionResultData[9], tMissionResultData[10])
	
	-- ===================================================================================================================================================
	--		Process Mission Results Popup Data
			
	if (tMissionResultData[5] == NO) or (tMissionResultData[5] == PREVENTED) then			-- Mission failed ([5] is mission result), display popup
		iTopIcon = 3																		-- Red icon, blue bg		
		sReportTitle = "Covert Opertation: " .. g_MissionList[iMissionNumber][ML_MISSION_NAME]	-- Text name of mission
		sReconHeader = "Results of your Covert Op near " .. sTargetCityName .. " conducted by " .. sSpyName	-- Assign the major civ header			
		sReconReport = sLineSeperator .. sSeparatorString .. "                        [ICON_DENOUNCE]    MISSION FAILURE!!!    [ICON_DENOUNCE]" .. sSeparatorString .. sLineSeperator .. sSeparatorString .. tMissionResultData[3] .. sSeparatorString .. sLineSeperator .. sSeparatorString .. tMissionResultData[4]		-- Build report		
	else																					-- Mission succeeded.  If mission was "Recon" popup alread		
		if iMissionNumber == CONDUCT_RECON then												-- If mission isn't recon, generate a new title/header
			sReportTitle = "Reconaissance Report"
			sReconHeader = "Intelligence from agents near " .. sTargetCityName		-- Default header (city state mostly)
			if (not pTargetPlayer:IsMinorCiv()) and (not pTargetPlayer:IsBarbarian()) then			-- If target civ isn't city state and not barb, then...
				sReconHeader = "Intelligence concerning the " .. sTargetPlayerAdjective .. " from agents near " .. sTargetCityName	-- Assign the major civ header
			end
			sReconReport = sLineSeperator .. "[NEWLINE]" .. tMissionResultData[7] .. sSeparatorString .. sLineSeperator .. sSeparatorString .. tMissionResultData[4]

		else
			sReportTitle = "Covert Opertation: " .. g_MissionList[iMissionNumber][ML_MISSION_NAME]
			sReconHeader = "Results of your mission near " .. sTargetCityName .. " conducted by " .. sSpyName	-- Assign the major civ header			
			sReconReport = sLineSeperator .. sSeparatorString .. tMissionResultData[3] .. sSeparatorString .. sLineSeperator .. sSeparatorString .. tMissionResultData[4]			
						
		end		
	end

	--sReconReport = sReconReport .. sLineSeperator .. "[NEWLINE]" .. tMissionResultData[4]

	ProcessReconPopup(sReportTitle, sReconHeader, sReconReport, iTopIcon)					-- Display Recon Report popup

	g_Human_Last_Turn_CO_Launched = Game.GetGameTurn()										-- Set this game turn as the last turn CO launched	
	
	local iAttackingPlayerGold = pPlayer:GetGold()											-- Process mission costs; get attacking civ total gold on hand			
	--print("GOH: " .. iAttackingPlayerGold .. "    mission cost: " .. iMissionCost)
	iAttackingPlayerGold = iAttackingPlayerGold - iMissionCost								-- Get attacking civ gold on hand
	if iAttackingPlayerGold < 0 then iAttackingPlayerGold = 0 end							-- If attacking civ gold is LT 0, set at 0 (cannot allow a negative number)
	iAttackingPlayerGold = pPlayer:SetGold(iAttackingPlayerGold)							-- Set attacking civ total gold to previous amount less mission cost
	--print("New GOH: " .. iAttackingPlayerGold)
					
end
