-- Process_AI_Mission_Results.lua
-- Author: sman
-- DateCreated: 01/11/18 4:06:05 AM
--------------------------------------------------------------

-- =================================================================================
--			Globals

gAiMissionTable = {}																		-- Master table holding all AI covert missions - both open and closed
	
-- =================================================================================
--			Functions

function OpenMissionStats (pPlayer)															-- Determine how many open missions pPlayer has, and what game turn the most recent mission is supposed to occur
	local iMissionCount = 0
	local iLatestMissionTurn = -1
	local pPlayerID = pPlayer:GetID()	
	if #gAiMissionTable > 0 then															-- If there are any missions in the table, then...
		for iIndex = 1, #gAiMissionTable do													-- Cycle through all the missions
			if (gAiMissionTable[iIndex][1] == pPlayerID) and (gAiMissionTable[iIndex][2] == YES) then		-- Does this mission belong to pPlayerID and is it still open?
				iMissionCount = iMissionCount + 1											-- Bump the counter for pPlayer
				if gAiMissionTable[iIndex][5] > iLatestMissionTurn then						-- If the "turn to execute" is higher than our previous high turn to execute, then...
					iLatestMissionTurn = gAiMissionTable[iIndex][5]							-- Make this mission the "latest executing mission scheduled for pPlayer"
				end
			end
		end
	end
	--print ("Total missions: " .. #gAiMissionTable .. "    total missions for player " .. pPlayerID .. ":  " .. iMissionCount)
	return iMissionCount, iLatestMissionTurn												-- Return total # of open missions for pPlayer and what is the latest turn one of their missions will execute
end


function GetTargetPlayer(pPlayer)															-- Select a target civ for this operation; return target ID or -1 if no valid targets
	local iTargetPlayerID = -1																-- Evaluates a set of fixed criteria to determine the relationship between the attacking civ and all other civs (human too)
	local pPlayerTeam = Teams[pPlayer:GetTeam()]											-- Then adjusts those criteria by a random +- 25%.  Civ with the "worst" relationship is the target
	local iWorstEnemyCount = 0	
	--print("Getting a target player for player: " .. pPlayer:GetID())
	
	if math.random(1, 100) < 33 then														-- Get a truly random civ to attack (33% of the time)
		local tLivingCivs = {}
		for iPlayer=0, GameDefines.MAX_MAJOR_CIVS-1 do										-- Cycle through all major players - everyone should be alive at this point (game start)		
			local pTempTeam = Teams[Players[iPlayer]:GetTeam()]								-- Get team pointer for loop player

			if pTempTeam:IsHasMet(pPlayer:GetTeam()) and									-- Only consider civs we've met
					(Players[iPlayer]:IsAlive()) and										-- Only consider living teams
					(iPlayer ~= pPlayer:GetID()) then										-- Only consider civs who are not ourselves		
						
				local iPlayerScore = Game.GetPlayerScore(iPlayer) or 0						-- CHANGE: added highest game score as a potential determinant for target (sman:20180209)				
				tLivingCivs[#tLivingCivs+1] = { [1] = Players[iPlayer]:GetID(), [2] = iPlayerScore }	-- Add this player ID to the table of possible targets and current score				
			end
		end
		if #tLivingCivs > 0 then															-- If there are 1 or more potential targets, then
			if math.random(1, 100) < 33 then												-- 33:67 chance the target will be truly random, or based on highest game score of known civs;  true random is a third of a third, or 11% actual total chance (sman:20180209)
				iTargetPlayerID = tLivingCivs[math.random(1, #tLivingCivs)][1]				-- Worst enemy is an equally random/living/met major civ
			else																			-- New process for targeting known civ with highest score (sman:20180209)
				iHiScoreIDX = 1																-- Assume 1st Civ in table is hi score
				for iDX = 1, #tLivingCivs do												-- Scroll through all civs in table (all known civs)
					if tLivingCivs[iDX][2] > tLivingCivs[iHiScoreIDX][2] then				-- If new civ score is higher, then
						iHiScoreIDX = iDX													-- Make this civ the new Hi scorer/target
					end
				end
				iTargetPlayerID = tLivingCivs[iHiScoreIDX][1]								-- Make the actual taget civ the highest scorer we found				
			end				
		else																				-- Any issues means the function will return -1 and no enemy is selected
			iTargetPlayerID = -1
		end
	else																					-- Use a formula/process to determine worst enemy
		for iPlayer=0, GameDefines.MAX_MAJOR_CIVS-1 do										-- Cycle through all major players - everyone should be alive at this point (game start)		
			local pTempTeam = Teams[Players[iPlayer]:GetTeam()]								-- Get team pointer for loop player
			if pTempTeam:IsHasMet(pPlayer:GetTeam()) and									-- Only consider civs we've met
					(Players[iPlayer]:IsAlive()) and										-- Only consider living teams
					(iPlayer ~= pPlayer:GetID()) then										-- Only consider civs who are not ourselves				
				local iBadnessCount = 0														-- Badness is a summary of all the things the target civ has done against pPlayer
				local iEnemyTeam = Players[iPlayer]:GetTeam()								-- Some methods need target team ID instead of target player ID

				if pPlayer:IsCapitalCapturedBy(Players[iPlayer]:GetID()) then				-- They have our capital...
					iBadnessCount = iBadnessCount + 20
				end

				if (pPlayer:IsNukedBy(Players[iPlayer]:GetID())) then						-- The nuked us in the past...
					iBadnessCount = iBadnessCount + 10
				end

				if pPlayerTeam:IsAtWar(iEnemyTeam) then										-- We are at war with them...
					iBadnessCount = iBadnessCount + 10
				end

				if Players[iPlayer]:IsDenouncingPlayer(pPlayer:GetID()) then				-- They are denouncing us...
					iBadnessCount = iBadnessCount + 7
				end

				if pPlayer:IsPlayerRecklessExpander(Players[iPlayer]:GetID()) then			-- They are reckless expanders...
					iBadnessCount = iBadnessCount + 5
				end

				if Players[iPlayer]:IsHasPlayerBulliedProtectedMinor() then					-- They have bullied protected city states...
					iBadnessCount = iBadnessCount + 2
				end

				if Players[iPlayer]:IsEmpireUnhappy() then									-- They are unhappy...
					iBadnessCount = iBadnessCount + 1
				end
				if Players[iPlayer]:IsEmpireVeryUnhappy() then								-- They are very unhappy...
					iBadnessCount = iBadnessCount + 2
				end
				if Players[iPlayer]:IsEmpireSuperUnhappy() then								-- They are super unhappy...
					iBadnessCount = iBadnessCount + 4
				end
			
				local iPlayerPower = pPlayer:GetPower()										-- Evaluate relative strength.  Get pPlayer's "power"
				local iTargetPlayerPower = Players[iPlayer]:GetPower()						-- Get Target player's "power"
				local iPowerRatio = math.floor(iTargetPlayerPower / iPlayerPower)			-- Get the ratio of strength
				if iPowerRatio < 1 then iPowerRatio = math.floor(iPlayerPower / iTargetPlayerPower) * -1 end	-- If enemy is stronger, then reverse the ratio and make it negative
				if iPowerRatio > 3 then iPowerRatio = 3 end									-- After 3 times larger, it doesn't matter
				if iPowerRatio < -3 then iPowerRatio = -3 end								-- After 3 times smaller, it doesn't matter

				iBadnessCount = iBadnessCount + (iPowerRatio * 4)							-- Change iBadnessCount: if we are stronger, add the Ratio*4; if they are stronger, subtract the reversered ration by Ratio*4; max is +-12 points

				local iAdjustment = (math.random(1, 50) - 25) + 100							-- Add some randomness to the overall equation (+- 25%)			
				--print("Target civ: " .. iPlayer .. "   ibad: " .. iBadnessCount .. "    Adjustment: " .. iAdjustment)			
				iBadnessCount = math.floor (iBadnessCount * (iAdjustment/100) )				-- Adjust the overall badness bt +-25; provides some variety - especially for really "bad" enemies			
				
				if pPlayer:IsDoF(iPlayer) then												-- If iPlayer is DoF with pPlayer, cut chance in half (sman:20180213)
					iBadnessCount = math.floor (iBadnessCount / 2 )	
					--print("Cutting Badness due to DoF between players " .. pPlayer:GetID() .. " and " .. iPlayer)
				end
				
				if iBadnessCount > iWorstEnemyCount then									-- if iPlayer's badness is worse than our previous "worst enemy" then...
					iTargetPlayerID = Players[iPlayer]:GetID()								-- We have a new worst enemy
					iWorstEnemyCount = iBadnessCount
				end
			end
		end
	end
	--print("Worst enemy/target: " .. iTargetPlayerID)
	return iTargetPlayerID																	-- Returns -1 if no civs can be attacked - not sure how this could happen, though
end


function BuildWarningString(pPlayer, iGameTurn, iIndex, iEspionagePosture, bTrueWarning)	-- Builds a warning message for Notifications; adds ambiguity to a report based on pPlayers Espionage Posture
																							-- Last argument="Is this a warning" if true, then future/impending report against human player (RUMINT); if false then past reporting (FIR)
	local sWarnString = ""																	-- Return warning message value	
	local sFadeString = ""																	-- Return fade value
	
	local sRumorReport = "Operation results unknown."										-- What is the default report value?
	local sRumorReporter = "unknown"														-- Who is issuing the report?
	local sRumorAttacker = "unknown"														-- Who is attacking in the report?
	local sRumorLocation = "a distant land"													-- Where did they attack?  City?  Civ?	
	local sRumorTurn = "recently"															-- Game turn mission executed	
	local iReportQuality = 0
	
	local sActualReporter = Players[gAiMissionTable[iIndex][7]]:GetCivilizationAdjective()	-- Adjective of actual target civ;  [7] = iTargetPlayerID
	local sActualTargetCiv = Players[gAiMissionTable[iIndex][7]]:GetCivilizationShortDescription() or "unknown civ"	-- Name of actual target civ;  [7] = iTargetPlayerID
	local sActualTargetCity = gAiMissionTable[iIndex][8]:GetName() or "unknown city"		-- Name of actual target city;  [8] = target city pointer
	local sActualAttacker = Players[gAiMissionTable[iIndex][1]]:GetCivilizationAdjective() or "unknown attacker"	-- Adjective of actual target civ;  [1] = mission owner PlayerID
	
	local pCapCity = pPlayer:GetCapitalCity()												-- Get the capital city for pPlayer 	
	local iX = pCapCity:GetX()																-- Default the notification X to the pPlayer capital city X
	local iY = pCapCity:GetY()																-- Default the notification Y to the capital city Y	
					
	local iReportStrenth = iEspionagePosture + (math.random(20, 60) - 30)					-- Strength of pPlayer Espionage Posture: -10..+30  (changed 20180215:sman)
	if iReportStrenth < 10 then iReportStrenth = 10	end										-- Minimum chance of getting good report is 10%
	if bTrueWarning then																	-- For "RUMINT/true warnings" (attacks against the human player) the upper confidence bound is higher
		if iReportStrenth > 85 then iReportStrenth = 85 end									-- Maximum chance of getting good report is 80% for true warnings; changed to 85% (sman:20180209)
	else																					-- For not "FIR: true warnings" the confidence limit is lower
		if iReportStrenth > 66 then iReportStrenth = 66 end									-- Maximum chance of getting good report is 50% for normal reports; changed to 66% (sman:20190209)
	end

	-- Determine the specificity of the report, based on the iReportStrength
	
	if math.random(1, 100) < iReportStrenth then											-- Chance of WHO reported; defaults to "unk"
		sRumorReporter = sActualReporter													-- Change WHO reported to the actual target civ
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end

	if math.random(1, 100) < iReportStrenth then											-- Chance of WHO attacked; defaults to "unk"
		sRumorAttacker = sActualAttacker													-- Change WHO attacked to the actual attacking civ
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end

	if math.random(1, 100) < iReportStrenth then											-- Chance of WHERE attacked; defaults to "distand land"
		if math.random(1, 100) < iReportStrenth  then										-- Determine if reporting City or Civ
			sRumorLocation = sActualTargetCity												-- Return the city
			if (gAiMissionTable[iIndex][8]) then											-- If the target city exists (i.e. not captured, razed, etc.), then; only chance Notification location if we actually know the city
				iX = gAiMissionTable[iIndex][8]:GetX()										-- Set the notification X to the target city X
				iY = gAiMissionTable[iIndex][8]:GetY()										-- Set the notification Y to the target city Y
			end
		else
			sRumorLocation = sActualTargetCiv												-- Return the civ
		end
		if not sRumorLocation then sRumorLocation = sActualTargetCiv end					-- If for some reason the location is nil, return the civ		
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end

	if math.random(1, 100) < iReportStrenth then											-- Chance of actual report; defaults to "Results unk"
		sRumorReport = gAiMissionTable[iIndex][9]											-- Change report to the actual report
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end

	if math.random(1, 100) < iReportStrenth then											-- Chance of actual mission turn; defaults to current game turn
		local iAdjustTurn = math.random(1, 3) - 2											-- Adjust the turn +- 1
		sRumorTurn = "on game turn " .. gAiMissionTable[iIndex][5] + iAdjustTurn			-- Change to the actual mission turn		
		iReportQuality = iReportQuality + 1													-- Increase report quality
	end
	
	if bTrueWarning then																	-- RUMINT: Adjust the warning ambiguity to more details that a home-civ should have	
		local sMissionType = "unknown"														-- Default mission type to report
		local iMissionChances = math.random(1, 100)											-- Chance of discovering exactly what the enemy is up to
		if iMissionChances < math.floor(iReportStrenth / 2) then							-- The iMissionsChance is less than half of the successful report number
			sMissionType = "'" .. g_MissionList[gAiMissionTable[iIndex][6]][ML_MISSION_NAME] .. "'"		-- Get the exact name of impending attack
		elseif 	iMissionChances < iReportStrenth then
			sMissionType = "Level " .. g_MissionList[gAiMissionTable[iIndex][6]][ML_MISSION_LEVEL]		-- Returns the level of the planned attack
		end
		
		if sRumorLocation == "a distant land" then sRumorLocation = sActualTargetCiv end	-- Ensure the target location is either a city or civ; no "unknowns" allowed
		if sRumorTurn == "recently"	then sRumorTurn = "soon" end							-- If we don't have specific game turn, change default to a future time
		
		sWarnString = "RUMINT: Your Counter-Intelligence services report " .. sRumorAttacker .. " agents are planning to conduct a " .. sMissionType .. " Covert Operation against " .. sRumorLocation .. " " .. sRumorTurn .. "."
		sFadeString = "RUMINT: Possible Covert Op threatened against " .. sRumorLocation
	else																					-- FIR: If not a "true" warning, insert "ambiguity" into the report
		sWarnString ="FIR: Reports by " .. sRumorReporter .. " sources state that " .. sRumorTurn .. ", " .. sRumorAttacker .. " foreign agents conducted a Covert Operation against " .. sRumorLocation .. "."
		sWarnString = sWarnString .. " " .. sRumorReport
		sFadeString = "FIR: " .. sRumorAttacker .. " agents conduct Covert Operation against " .. sRumorLocation
	end
	return sWarnString, sFadeString, iX, iY, iReportQuality									-- Report quality is the # of specific items in the report (max = 5)
end


function Component1_Schedule_Mission(pPlayer, iGameTurn, iCurrentEra)						-- Determines if pPlayer can schedule a mission this turn
	local ipPlayerID = pPlayer:GetID()														-- Gets the player ID of pPlayer
	local iOpenMissionCount, iLatestMissionTurn = OpenMissionStats (pPlayer)				-- Get # of open missions and latest mission date for this player
		
	if iCurrentEra < ERA_CLASSICAL_ID then return end										-- No missions allowed until in Classical Era (sman:20180211)

	if iOpenMissionCount >= (iCurrentEra + 1) then return end								-- Can never have more open missions than the Era+1 you're in
	if iOpenMissionCount >= g_Open_Mission_Limit then return end							-- Only let pPlayer have g_Open_Mission_Limit open missions at a time, which is adjusted for difficulty/gamespeed
	if (iGameTurn > 20) and (iLatestMissionTurn > (iGameTurn - g_AI_Minimum_Turn_Interval)) then return end		-- Force a g_AI_Minimum_Turn_Interval or more game turn wait between missions, adjusted for difficulty/gamespeed
				
	local iToAttackOrNotToAttack = math.random(1, 100)										-- Chance of scheduling an Attack
	
	--print("================================= player id: " .. pPlayer:GetID())	
	if ipPlayerID ==  0 then ipPlayerID = 99 end											-- Set index to 99 for the human player - should only hit during Live Tuner games
	if iToAttackOrNotToAttack > g_Leader_Stats[ipPlayerID][COVERT_OPERATIONS_PROPENSITY] then return end		-- If random % is greater than COP threashold, exit early; otherwise SCHEDULE!
			
	local iTargetPlayerID = GetTargetPlayer(pPlayer)										-- Determine Target Civ - Who do we hate the most?
	if iTargetPlayerID == -1 then return end												-- If no valid civs to attack, exit early
			
	local iTurnToExecute = iGameTurn + math.random(8, 12)									-- Determine the actual turn to launch the mission			
	local iTurnToWarn = iTurnToExecute - math.random(0, 5)									-- Higher bound here needs to be 1 less than the lower bound of iTurnToExecute!
	if iTurnToWarn == iTurnToExecute then iTurnToWarn = -1 end								-- Sometimes, you don't get a warning...
		
	local iIndex = #gAiMissionTable + 1														-- New row index in the Mission_Table
	if iIndex == 0 then iIndex = 1 end														-- Ensure table index starts no lower than 1
		
	local bIsCapitalOK = true																-- Default to let capitals be valid targets for covert ops
	if iMissionNumber == STEAL_CITY then bIsCapitalOK = false end							-- If the mission is Steal City, then the capital is not a valid target
	local pTargetCity = GetRandomCity(Players[iTargetPlayerID], bIsCapitalOK)				-- Arguments: pPlayer, bIsCapitalOK
	if not pTargetCity then return end														-- If nil, no available cities (e.g. steal city and target civ has only capital?) - return early
		
	local iMissionNumber, iMissionCost = GetMostLikelyAIMission (pPlayer, iCurrentEra, iTargetPlayerID, pTargetCity)	-- Will return a wighted, valid mission between 1..14, depending on pPlayer Era and affordable
	
	if iMissionNumber ~= -1 then															-- Mission num should be 2 or higher; otherwise, AI has no valid missions and failed to schedule this opportunity, timer still starts over
		-- gAiMissionTable[iIndex][1] = attacking player ID    [2] = Is the mission open?  Will change to NO once the mission is completed    [3] = What turn was mission scheduled?
		--		[4] = What turn to warn?    [5] = What turn to execute?   [6] = What mission type will we launch?    [7] = Who is the target player(ID)?    
		--		[8] = Target City pointer   [9] = Placeholder element - used to store mission results after the mission - for future reference, [10] = Mission Cost, [11]=International response 
		gAiMissionTable[iIndex] = { [1] = pPlayer:GetID(), [2] = YES, [3] = iGameTurn, [4] = iTurnToWarn, [5] = iTurnToExecute, [6] = iMissionNumber, [7] = iTargetPlayerID, [8] = pTargetCity, [9] = "", [10] = iMissionCost, [11] = ""}
		--print("Scheduling a mission. Index: " .. iIndex .. "    Player: " .. pPlayer:GetID() .. "  on game turn: " .. iGameTurn .. "    warning turn: " .. iTurnToWarn .. "  planned turn: " .. iTurnToExecute .. "    target civ: " .. iTargetPlayerID ..  "    target city: " .. pTargetCity:GetName() .. "    iMissionNumber: " .. iMissionNumber .. "    iMissionCost: " .. iMissionCost)
	end

end


function Component2_Warn_Mission(pPlayer, iGameTurn, iCurrentEra)							-- Determines if a Warning Notification is required this turn - only for human player!
	-- There is a small chance of receiving a report about covert operation directed against another civ.  Chance is based on the civ capital's EspionagePosture
	-- This is why the EP is checked every turn for the human player - in case the EP improves (more buildings, units, spies, etc.)
	local iEspionagePosture = CalculateEspionagePosture(pPlayer, pPlayer:GetCapitalCity())	-- Get pPlayer's EP, i.e. how competent pPlayer's intel apparatus is
	local sWarnString = ""
	local sFadeString = ""
	local iReportQuality = 0
	local iX = -1
	local iY = -1
	
	-- Possibility to view foreign Int report
	local iChanceToView = math.floor(iEspionagePosture / 3)									-- Original EP / 4 = chance to view foreign report; changed to EP/3, better chance of details (sman:20180215)
	if iChanceToView < 10 then iChanceToView = 10											-- Minimum chance to view is 2%; changed to 10% (sman:20180209)
	elseif iChanceToView > (12 + (iCurrentEra*3)) then iChanceToView = 12 + (iCurrentEra*3) end	-- Maximum chance to view is 8% + (Current Era*2); Information Era = 7 (22%); Changed to 12+(Era*2), or 15..33% (sman:20180209)

	if math.random(1, 100) < iChanceToView then												-- Can we see the report?
		for iIndex = #gAiMissionTable, 1, -1 do												-- Cycle through all missions in the table backwards - looking for first closed mission that isn't directed at pPlayer
			if (gAiMissionTable[iIndex][2] == NO) and (gAiMissionTable[iIndex][7] ~= pPlayer:GetID()) then	-- Is the mission CLOSED?  Is the target NOT a human?  If so, grab the most recent report										
				sWarnString, sFadeString, iX, iY, iReportQuality = BuildWarningString(pPlayer, iGameTurn, iIndex, iEspionagePosture, false)	-- last argument="Warning" if true, then future/impending report; if false then past reporting
				if iReportQuality >= 2 then													-- Message has to have 2 or more elements "correct" otherwise, abandon
					SendNotification(4, sWarnString, sFadeString, nil, iX, iY)				-- Issue Notification
					break																	-- Exit the loop early
				end				
			end
		end																					-- If the loop terminates normally, we didn't find any CLOSED missions targeting non-human players
	end																						-- If no missions found, no notification sent, continue on with processing...
	
	-- Possibility to recieve warning about a mission targeting the human player
	for iIndex = 1, #gAiMissionTable do														-- Cycle through all missions in the table		
		if (gAiMissionTable[iIndex][2] == YES) and (gAiMissionTable[iIndex][7] == pPlayer:GetID()) and (gAiMissionTable[iIndex][4] == iGameTurn) then	-- Open mission, target human, warn turn = game turn					
			sWarnString, sFadeString, iX, iY, iReportQuality = BuildWarningString(pPlayer, iGameTurn, iIndex, iEspionagePosture, true)	-- last argument="Warning" if true, then future/impending report; if false then past reporting									
			SendNotification(4, sWarnString, sFadeString, nil, iX, iY)						-- Issue Notification
		end
	end
end

function Component3_Execute_Mission(pPlayer, iGameTurn, iCurrentEra)						-- Actually executes a previously-scheduled mission
	local pUnit = nil																		-- AI doesn't use units to launch Covert Ops
		
	for iIndex = 1, #gAiMissionTable do													-- Cycle through all missions

		if (gAiMissionTable[iIndex][1] == pPlayer:GetID()) and (gAiMissionTable[iIndex][5] >= iGameTurn) and (gAiMissionTable[iIndex][2] == YES) and (pPlayer:IsAlive()) then		-- pPlayer:GetID() has a mission scheduled for this turn, if the mission is OPEN, and is still alive?  Changed game turn test from "==" to ">=" in case for some reason the mission doesn't execute on time (sman:20180214)

			-- Mission results:		    Notify Type	FadeStr	Mission Result	Reaction	Msn		Incident	Summary		UnitSpawnType	Notify X,		Notify Y	---- default values for mission results data table
			--							1			2		3				4			5		6			7			8,				9,				10
			local tMissionResultData = { 1,			"",		"",				"",			1,		1,			"",			-1,				-1,				-1 }		-- Set up results table for passing around	
			local pTargetPlayer = Players[gAiMissionTable[iIndex][7]]						-- Set the target player pointer
			local iMissionNumber = gAiMissionTable[iIndex][6]								-- Set the mission number
			local pTargetCity = gAiMissionTable[iIndex][8]									-- Get the target city pointer
			
			if pTargetCity then																-- If pTargetCity is valid, then					
				local sAttckingAgentName, iSelectedAgentRank = GetAICivSpymaster(pPlayer)	-- Use "Spymaster" since AI may not have a spy in target city at the time; a bit of an advantage...
											
				GetMissionResult(tMissionResultData, iMissionNumber, pPlayer, sAttckingAgentName, sSelectedAgentRank, iCurrentEra, pTargetPlayer, pTargetCity)	-- Get results of AI CO mission
								
				GetInternationalResponses(tMissionResultData, iMissionNumber, pPlayer, pTargetPlayer, pTargetCity, sAttckingAgentName, sSelectedAgentRank)		-- Get the international response
				
				local iAiPlayerGold = pPlayer:GetGold()										-- Get total gold AI civ has on hand	
				--print("AI Civ GOH: " .. iAiPlayerGold .. "    mission cost: " .. gAiMissionTable[iIndex][10])					
				iAiPlayerGold = iAiPlayerGold - gAiMissionTable[iIndex][10]					-- Subtract the mission cost from AI player's gold on hand
				if iAiPlayerGold < 0 then iAiPlayerGold = 0 end								-- Prevent negative gold balance					
				pPlayer:SetGold(iAiPlayerGold)												-- Set attacking civ total gold to previous amount less mission cost
				--print("New AI Civ GOH: " .. pPlayer:GetGold() )


				if pTargetPlayer:IsHuman() then												-- Only build/display Notification if the human player is the AI CO target
					local sAttackerInfo = "an unidentified civilization"					-- Default attacker ID to report
					local sAppearenceInfo = "appear to be"									-- Default "certainty"
					local iEspionagePosture = CalculateEspionagePosture(pPlayer, pTargetCity) or 0		-- Target civ's EP (0..50 + (# Spies * 2)
					local iBuildingInfluence = GetBuildingInfluence(pTargetCity) or 0		-- Target City defensive buildings (0..50)
					local iUnitInfluence = GetUnitInfluence(pTargetPlayer, pTargetCity) or 0				-- # units*1 and Level of Covert Op units * 3

					iEspionagePosture =  math.floor(iEspionagePosture / (pPlayer:GetNumCities() + 1))	-- Determine how "thin" the resources are stretched; no div by 0 here...
					local iChanceToIDAttacker = math.floor(iEspionagePosture + (iBuildingInfluence / 3) + iUnitInfluence)	-- Chance to ID attacker
					
					if math.random(1, 100) < iChanceToIDAttacker then						-- Is the random # less than the chance to ID threshold?  Then,
						sAttackerInfo = pPlayer:GetCivilizationDescription()				-- Set the ID string from unid'd to id'd
						if math.random(1, 100) < iChanceToIDAttacker then sAppearenceInfo = "have been identified, coming" end	-- Check to see if we're certain or guessing
					end
										
					local sAttackerOrigin = "The perpetrators " .. sAppearenceInfo .. " from the " .. sAttackerInfo .. "."		-- Set the OP Fate element to include in the Notification
										
					local sSeparatorString = "[NEWLINE][NEWLINE]"									-- Experiment with how to separate message components
					--local sSeparatorString = "  "
					local sLongMessage = tMissionResultData[3]										-- Build the long message string piece by piece; [3]=Mission Result

					if sAttackerOrigin ~= "" then sLongMessage = sLongMessage .. sSeparatorString .. sAttackerOrigin end				-- Attacker ID string
					if tMissionResultData[4] ~= "" then sLongMessage = sLongMessage .. sSeparatorString .. tMissionResultData[4] end	-- International result string
					
					local iTargetCityX = pTargetCity:GetX() or -1											-- Get X location for target city
					local iTargetCityY = pTargetCity:GetY() or -1											-- Get Y location for target city	

					SendNotification(tMissionResultData[1], sLongMessage, tMissionResultData[2], pUnit, iTargetCityX, iTargetCityY)	-- Issue Notification
				end					
				gAiMissionTable[iIndex][9] = tMissionResultData[3] .. " " .. tMissionResultData[4] or "Status unknown"		-- Use the fade string as the summary message
			else																			-- pTargetCity no longer valid - city doesn't exist or is captured (therefore new pointer)			
				gAiMissionTable[iIndex][9] = "Status unknown"								-- Use the fade string as the summary message; may need to look here further
			end
			gAiMissionTable[iIndex][2] = NO													-- Show the mission as completed (Active mission? NO)
			--print("Executing a mission. Mission Index: " .. iIndex .. "    Player: " .. pPlayer:GetID() .. "  on game turn: " .. iGameTurn .. "  Planned Turn: " .. gAiMissionTable[iIndex][3] .. "    Target Civ: " .. gAiMissionTable[iIndex][7] .. "    Target City: " .. gAiMissionTable[iIndex][8]:GetName() .. "    Mission Number: " .. iMissionNumber .. "    Results: " .. gAiMissionTable[iIndex][9])

		end	
	end
end


function DisplayAIMissionTable(iGameTurn)													-- Display the contents of the AI Mission Table
	print("==============================================================================================================")
	print("AI Mission Table Contents.  Game Turn: " .. iGameTurn)
	if #gAiMissionTable == 0 then
		print("AI Mission Table is currently empty.")
	else
		for iIndex = 1, #gAiMissionTable do
			local sMissionStatus = "Open"
			if gAiMissionTable[iIndex][2] == NO then sMissionStatus = "Closed" end
			local sTempCityName = gAiMissionTable[iIndex][8]:GetName()
			if not sTempCityName then sTempCityName = "no longer valid" end
			
			--print("Msn #: " .. iIndex .. "   Attacker: " .. Players[gAiMissionTable[iIndex][1]]:GetName() .. "   Status: " .. sMissionStatus .. "   Scheduled Turn: " .. gAiMissionTable[iIndex][3] .. "   Launch Turn: " .. gAiMissionTable[iIndex][5] .. "   Defender: " .. Players[gAiMissionTable[iIndex][7]]:GetName() .. "   Tgt City: " .. sTempCityName .. "   Mission: " .. g_MissionList[gAiMissionTable[iIndex][6]][ML_MISSION_NAME] .. "   Results: " .. gAiMissionTable[iIndex][9])

			sString = string.format("Msn #: %5i    Mission: %-15.15s    Attacker: %-12.12s    Status: %-6s    Turn Scheduled: %5i    Launch Turn: %5i    Target: %-15.15s    Target City: %-15.15s    Results: %s",
				iIndex, g_MissionList[gAiMissionTable[iIndex][6]][ML_MISSION_NAME], Players[gAiMissionTable[iIndex][1]]:GetName(), sMissionStatus, 
				gAiMissionTable[iIndex][3], gAiMissionTable[iIndex][5], Players[gAiMissionTable[iIndex][7]]:GetName(), sTempCityName, gAiMissionTable[iIndex][9] )
			
			print(sString)

		end
	end
	print("==============================================================================================================")	
end


function DisplayCOMissionStats(iGameTurn)													-- Display mission success/fail/prevent stats
	print("==============================================================================================================")
	print("CO Mission Stats.  Game Turn: " .. iGameTurn)
	local iTotalAttempts, iTotalSuccess, iTotalSuccessRate, iTotalPrevented, iTotalPreventRate = 0, 0, 0, 0, 0
	for iIndex = 2, ABORT_MISSION-1 do		
		if (g_MissionList[iIndex]) then
			local iFails = g_MissionList[iIndex][ML_NUM_ATTEMPTS] - (g_MissionList[iIndex][ML_NUM_SUCCESS] + g_MissionList[iIndex][ML_NUM_PREVENTS])		
			local iSuccessRate = 0 
			if g_MissionList[iIndex][ML_NUM_ATTEMPTS] > 0 then
				iSuccessRate = math.floor( (g_MissionList[iIndex][ML_NUM_SUCCESS] / g_MissionList[iIndex][ML_NUM_ATTEMPTS]) * 100 )
			end

			sString = string.format("Msn: %-25.25s    Attempted: %6i    Successful: %5i (%3i %%)    Failed*: %5i    Prevented: %5i",
					g_MissionList[iIndex][ML_MISSION_NAME], g_MissionList[iIndex][ML_NUM_ATTEMPTS], g_MissionList[iIndex][ML_NUM_SUCCESS], iSuccessRate, iFails, g_MissionList[iIndex][ML_NUM_PREVENTS]  )   
			print(sString)			
		
			iTotalAttempts = iTotalAttempts + g_MissionList[iIndex][ML_NUM_ATTEMPTS]
			iTotalSuccess = iTotalSuccess + g_MissionList[iIndex][ML_NUM_SUCCESS]
			iTotalPrevented = iTotalPrevented + g_MissionList[iIndex][ML_NUM_PREVENTS]
		end
	end	
	print("-----")
	if iTotalAttempts > 0 then
		iTotalSuccessRate = math.floor ( ( iTotalSuccess / iTotalAttempts ) * 100	)
		iTotalPreventRate = math.floor ( ( iTotalPrevented / iTotalAttempts ) * 100	)
	end

	print("Total Attempts: " .. iTotalAttempts .. "    Total Success: " .. iTotalSuccess .. "  ( " .. iTotalSuccessRate .. "% )    Total Prevented: " .. iTotalPrevented .. "  ( " .. iTotalPreventRate .. "% )" )
	print("==============================================================================================================")	
end


-- =================================================================================
--			Process AI Covert Missions
--
--  Add function that drives AI CO missions - AI can use all missions except #1 (Conduct Recon)
-- 
-- =================================================================================

function PlayerDoTurn_ProcessAI_CovertOperations( iPlayer )									-- Only major AI civs can use covert missions.  Change to let city states launch?
	local pPlayer = Players[iPlayer]														-- Get pPlayer from player ID				
	local iGameTurn = Game.GetGameTurn()													-- Get current game turn
	local iCurrentEra = pPlayer:GetCurrentEra()												-- Get current player's Era level

	-- =========================================================================================================================
	-- Process AI Covert Operation Mission Components
	
	if (pPlayer:IsAlive()) and (not pPlayer:IsHuman()) and (not pPlayer:IsMinorCiv()) and (not pPlayer:IsBarbarian()) then	-- Process AI covert operations only for living major AI civs	
	
		Component1_Schedule_Mission(pPlayer, iGameTurn, iCurrentEra)						-- AI Process Component 1: Schedule an AI CO Mission:  To launch or not to launch, that is the question...		
		Component3_Execute_Mission(pPlayer, iGameTurn, iCurrentEra)							-- AI Process Component 3: Process an AI CO Mission
	
	elseif pPlayer:IsHuman() then
		
		Component2_Warn_Mission(pPlayer, iGameTurn, iCurrentEra)							-- AI Process Component 2: Warn human player only of an Upcoming AI CO Mission		

	end

	if (iPlayer == 0) then
	--	print("Game Turn: " .. iGameTurn)
		if (iGameTurn % 100 == 0) then														-- Display the AI mission table/mission stats tables - used during LiveTuner
	--if (pPlayer:IsHuman()) and (iGameTurn % 5 == 0) then										-- Display the AI mission table/mission stats tables - used during normal human playing
			DisplayCOMissionStats(iGameTurn)												-- Display Mission stats
			--DisplayAIMissionTable(iGameTurn)												-- Display AI Mission Table - printing this often leads to CTDs for some reason...
		end
	end						
			
end
GameEvents.PlayerDoTurn.Add( PlayerDoTurn_ProcessAI_CovertOperations )
