-- Process_International_Response.lua
-- Author: sman
-- DateCreated: 01/08/18 9:19:38 AM
--------------------------------------------------------------

-- =================================================================================================================================
-- Reponse global data

-- Incidence Severity Enumerations:  Minor (1/2= 25%)  Denounce (3/4 = 45%)  War (5/6/7 = 25%)  Global War (8 = 5%)
RESPONSE_MUTED = 1																			-- 10%														
RESPONSE_MINOR = 2																			-- 15%
RESPONSE_DENOUNCE = 3																		-- 30%
RESPONSE_DENOUNCE_WITH_ALLIES = 4															--  5%
RESPONSE_WAR = 5																			-- 15%
RESPONSE_WAR_WITH_ALLIES = 6																--  5%
RESPONSE_WAR_PERMANENT = 7																	-- 15%
RESPONSE_WAR_GLOBAL = 8																		--  5%

g_tIncidenceResponse = {  [RESPONSE_MUTED] = 1, [RESPONSE_MINOR] = 10, [RESPONSE_DENOUNCE] = 25, [RESPONSE_DENOUNCE_WITH_ALLIES] = 55, [RESPONSE_WAR] = 60, 
							[RESPONSE_WAR_WITH_ALLIES] = 75, [RESPONSE_WAR_PERMANENT] = 80, [RESPONSE_WAR_GLOBAL] = 95	}

g_iDomesticResponseProbability = 33															-- Percent chance of domestic resistance to Covert Ops; adjusted later by excess happiness
g_iAlliedReponseProbability = 66															-- Percent chance allies will denounce or declare war as a response to an incident
g_iGlobalWarResponseProbability = 50														-- Percent chance any civ will declare war as a result of an incident when RESPONSE_WAR_GLOBAL is selected
g_iMinimumChanceOfIncident = 10																-- Percent chance floor of any response for any mission


-- =================================================================================================================================
-- Functions

function GetIncidentOutcome(iMissionNumber, pPlayer, pTargetPlayer, pTargetCity)			-- Determines what happened during the covert mission (successful? operator survive? incident?)
	local iIncidentResult = NO																-- Default result is there was no incident	

	-- Return "ATWAR" response if target player and active player already at war; otherwise test for incident based on formula
	local pTargetTeam = Teams[pTargetPlayer:GetTeam()]										-- Get team instance for target civ
	local iAttackingTeamID = pPlayer:GetTeam()												-- Get Team ID for the attacking civ	
	if  pTargetTeam:IsAtWar(iAttackingTeamID) then											-- Are teams at war?  Then...
		iIncidentResult = ATWAR																-- Change return value to ATWAR
	else																					-- If not at war, then...		
		local iAttackerEspionagePosture = CalculateEspionagePosture(pPlayer, pTargetCity)	-- Get the EP for the attacker in target city
		local iDefenderEspionagePosture = CalculateEspionagePosture(pTargetPlayer, pTargetCity)	-- Get the EP for the defender in target city
		local iAdjustedIncidentChance = g_MissionList[iMissionNumber][ML_CHANCE_INCIDENT] + iDefenderEspionagePosture - iAttackerEspionagePosture	-- Basic chance of incident - Defense IP + Attack IP
		
		if iAdjustedIncidentChance < g_iMinimumChanceOfIncident then iAdjustedIncidentChance = g_iMinimumChanceOfIncident end
		local iIncidentChance = math.random(1, 100)
		if iIncidentChance < iAdjustedIncidentChance then									-- Did mission cause Incident?  Based on chance for specific mission
			iIncidentResult = YES															-- Change default return value 
		end
		--print("====> Incident Results: " .. iIncidentResult .. "    Basic chance of incident: " .. g_MissionList[iMissionNumber][ML_CHANCE_INCIDENT] .. "    Attack EP: " .. iAttackerEspionagePosture .. "    Defense IP: " .. iDefenderEspionagePosture .. "    IadjIncChance: " .. iAdjustedIncidentChance .. "    Incident Chance: " .. iIncidentChance)
	end	
	return iIncidentResult
end


function CheckDomesticResponse(pPlayer, IncidentResult)
	local iResponse = NO																	-- Default return to "NO" (domestic response)
	local iResponseFloor = 15																-- Minimum chance of a domestic response
	if IncidentResult == ATWAR then iResponseFloor = 5 end									-- Lower minimum chance if at war with target

	local iResponseChance = g_iDomesticResponseProbability - pPlayer:GetExcessHappiness()	-- Lower the chance of response by excess happiness; g_iDomesticChance=33
	if iResponseChance < iResponseFloor then iResponseChance = iResponseFloor end			-- Ensure at least xx% (iResponseFloor) chance of response
	if math.random(1, 100) < iResponseChance then											-- Check random result against chance of response
		iResponse = YES																		-- If random number is under that chance, set return to YES
	end
	return iResponse																		-- Return result
end


-- ==========================================================================================================
-- Process Incident Results

function GetInternationalResponses(tMissionResultData, iMissionNumber, pPlayer, pTargetPlayer, pTargetCity)	-- Get the international response

	local IncidentResult = NO																-- Default response is NO
	local NotificationStringLong = ""														-- Clear the response message

	IncidentResult = GetIncidentOutcome(iMissionNumber, pPlayer, pTargetPlayer, pTargetCity)	-- Determine response YES/NO

	if IncidentResult ~= NO then															-- If YES or ATWAR, then there is an international incident to process
		local iUpperBound = 100
		local iResponseSeverity = math.random(1, iUpperBound)								-- Just how severe is the aftermath?						
		
		--print ("========> Incident Severity.  Upper bound: " .. iUpperBound .. "    Unadjusted Severity: " .. iResponseSeverity)

		-- Adjust to the severity based on mission level.  The higher level the mission, the higher the chance for a more serious response - these stack on top of normal severity number!
		-- Unsuccessful range: L1=0..0, L2=3..10, L3=6..20, L4=9..30, L5=12..40;   successful range: L1=0..0, L2=8..15, L3=16..30, L4=24..40, L5=32..50 
		local iLowerBound = 1																-- Unsuccessful mission lower bound of random impact range
		local iUpperBound  = 6																-- Unsuccessful mission upped bound of random impact range										
		if tMissionResultData[5] == YES then 												-- Increase random impact range for SUCCESSFUL missions		
			iLowerBound = 4																	
			iUpperBound = 10 			
		end
		
		local iSeverityAdjustment = math.random((g_MissionList[iMissionNumber][ML_MISSION_LEVEL] - 1) * iLowerBound, (g_MissionList[iMissionNumber][ML_MISSION_LEVEL] - 1) * iUpperBound)	-- CO ML_MISSION_LEVEL (1..5)
		iResponseSeverity = iResponseSeverity + iSeverityAdjustment
		
		if iMissionNumber == NUKE_SUITCASE then												-- Special adjustment for Suitcase Nuke mission		(sman:20180210)
			iResponseSeverity = iResponseSeverity * 2										-- Double the severity that would be otherwise normal
			--print("Severity doubled!  New amount: " .. iResponseSeverity)
		end

		if (g_MissionList[iMissionNumber][ML_MISSION_LEVEL] < 4) and (iResponseSeverity >= g_tIncidenceResponse[RESPONSE_WAR_WITH_ALLIES]) then	-- Prevent wider war for less impacting COs (Levels 1..3); level 4&5 still cause wider war
			iResponseSeverity = g_tIncidenceResponse[RESPONSE_WAR_WITH_ALLIES] - 1								-- If response is War W/ Allies or higher, drop it to simple DoW
		end
				
		local iIndex = #g_tIncidenceResponse												-- Set the index to the last element in the "incidence response" table	
		for idx = 1, #g_tIncidenceResponse - 1 do											-- Move through the incidence response table from frist element until next-to-last
			if (iResponseSeverity >= g_tIncidenceResponse[idx]) and (iResponseSeverity < g_tIncidenceResponse[idx+1]) then		-- If the score is between this element and the next, then
				iIndex = idx																-- Set the return index to the loop index value
				break																		-- Terminate the loop early, we've found what we're looking for
			end
		end			
		
		if pTargetPlayer:IsHuman() then														-- If target is human, don't allow forced denounce or DoW - leave it up to the player to decide
			if (iIndex == RESPONSE_DENOUNCE) or (iIndex == RESPONSE_WAR) or (iIndex == RESPONSE_WAR_PERMANENT) then		-- Test for denounce, DoW, or Perma DoW
				--print("Altered iIndex from " .. iIndex .. " to 2, since the target is a human player.")
				iIndex = RESPONSE_MINOR														-- Set the response to the generic "tut tut from the int'l community"
			end				
		end

		if (IncidentResult == ATWAR) and (iIndex == RESPONSE_DENOUNCE) then					-- If two civs already at war, change a denounce to a minor response (sman:20180215)
			iIndex = RESPONSE_MINOR 
		end					

		--print("========>Incident Severity.  Final Severity: " .. iResponseSeverity .. "    iIndex: " .. iIndex .. "   Upperbound: " .. iUpperBound .. "   Severity Adjust, based on mission level: " .. iSeverityAdjustment)
				
		-- ======================		Response: Muted ===================
		if iIndex == RESPONSE_MUTED then													-- Minor notification
			NotificationStringLong = NotificationStringLong .. "There has been a very muted international response to this incident."

		-- ======================		Response: Minor ===================
		elseif iIndex == RESPONSE_MINOR then												-- Minor notification, with possibility of brief resistance in the capital
			NotificationStringLong = NotificationStringLong .. "There has been only a minor international murmur about this incident."		
		
		-- ======================		Response: Denounce ===================
		elseif iIndex == RESPONSE_DENOUNCE then												-- Issue denunciation, with possibility of brief resistance in the capital (processed below)
			if (pTargetPlayer:GetID() < GameDefines.MAX_MAJOR_CIVS-1)  and					-- Only major civs can denounce; if we get here and target is City State, no "real" response will occur or be included in Notification string
					( not pTargetPlayer:IsHuman() )  then					
				NotificationStringLong = NotificationStringLong .. pTargetPlayer:GetCivilizationShortDescription() .. " has denounced " .. pPlayer:GetCivilizationShortDescription() .. " !"		
				pTargetPlayer:DoForceDenounce(pPlayer:GetID())								-- Force denunciation of target civ to the active civ		
			else																			-- Target city was City State, so general message, no other action; some possible domestic response further down		
				NotificationStringLong = NotificationStringLong .. "There has been minimal international response about this incident."			
			end
			
		-- ======================		Response: Team Denounce ===================
		elseif iIndex == RESPONSE_DENOUNCE_WITH_ALLIES then									-- Possible denunciation by target civ and all its DOF's, with possibility of brief resistance in the capital
																							-- Only non-City States can denounce!			
			local sTempString = "These Civilizations have denounced " .. pPlayer:GetCivilizationShortDescription() .. ":  "			
			local iDenouncingCivs = 0
			local iTargetPlayerID = pTargetPlayer:GetID()									-- Get the target civ's team ID
			
			for idx = 1, GameDefines.MAX_MAJOR_CIVS-1, 1 do									-- Loop through all major AI players (skip human = 0)				
				local pTempPlayer = Players[idx]											-- Get the loop player's instance
				local pTempTeam = Teams[pTempPlayer:GetTeam()]								-- Get team pointer for loop player
				
				if pTempPlayer:IsAlive() and												-- Only check civs still in the game
						 pTempTeam:IsHasMet(pPlayer:GetTeam())	and							-- Has this civ met the attacking player?  GetTeam returns Team ID
						( pTempPlayer:IsDoF(iTargetPlayerID) or iTargetPlayerID == idx ) and	-- Civ has DOF with target civ or is the target civ
						( pPlayer:GetID() ~= idx ) then										-- Target Civ cannot denounce against itself
					
					if math.random(1, 100) < g_iAlliedReponseProbability then				-- Random # below threshold (66% chance), allied civ will respond!
						pTempPlayer:DoForceDenounce(pPlayer:GetID())						-- Force the denunciation
						iDenouncingCivs = iDenouncingCivs + 1								-- Bump the denouncing civ counter
						sTempString = sTempString.. pTempPlayer:GetCivilizationShortDescription() .. "   "	-- Add the civ to the running string
					end
				end
			end
			if iDenouncingCivs == 0 then													-- If no one denounced, empty the string 			 
				sTempString = "There has been only a minor international murmur about this incident." 
			else
				--sTempString = sTempString .. "[NEWLINE]"
			end							
			NotificationStringLong = NotificationStringLong .. sTempString			
					
		-- ======================		Response: Target civ declares either War or Permanent War ===================
		elseif (iIndex == RESPONSE_WAR) or (iIndex == RESPONSE_WAR_PERMANENT) then			-- Target civ declares war, permanently in extreme cases; with possibility of brief resistance in the capital
			local sDuration = ""
			local pTargetTeam = Teams[pTargetPlayer:GetTeam()]								-- Get team pointer for target player
			local pPlayerTeam = Teams[pPlayer:GetTeam()]									-- Get team pointer for attacking player
			pTargetTeam:DeclareWar(pPlayerTeam:GetID())										-- Force DOW
			if iIndex == RESPONSE_WAR_PERMANENT then										-- If this is a permanent DOW, then
				pTargetTeam:SetPermanentWarPeace(pPlayerTeam:GetID(), true)					-- Set DOW to permanent
				sDuration = "PERMANENT "													-- Adjust duration description
			end
			NotificationStringLong = NotificationStringLong .. pTargetPlayer:GetCivilizationShortDescription() .. " has declared " .. sDuration .. "war on " .. pPlayer:GetCivilizationShortDescription() .. "!"		

		-- ======================		Response: Team War ===================
		elseif iIndex == RESPONSE_WAR_WITH_ALLIES then										-- Possible DOW's by target civ and all its DOF's, with possibility of brief resistance in the capital		
			local pPlayerTeam = Teams[pPlayer:GetTeam()]									-- Get team pointer for attacking player
			local sTempString = "These Civilizations have declared war on " .. pPlayer:GetCivilizationShortDescription() .. ":  "			
			local iDoWCivCount = 0
			local iTargetPlayerID = pTargetPlayer:GetID()									-- Get the target civ's team ID
			
			for idx = 1, GameDefines.MAX_MAJOR_CIVS-1, 1 do									-- Loop through all major AI players (skip human = 0)				
				local pTempPlayer = Players[idx]											-- Get the loop team's team ID
				local pTempTeam = Teams[pTempPlayer:GetTeam()]								-- Get team pointer for target player

				if pTempPlayer:IsAlive() and												-- Only test for civs still alive
						pTempTeam:IsHasMet(pPlayer:GetTeam()) and							-- Has this civ met the attacking player?  GetTeam returns Team ID
						( pTempPlayer:IsDoF(iTargetPlayerID)  or  iTargetPlayerID == idx ) and		-- Civ DOF with target civ or is the target civ
						( pPlayer:GetID() ~= idx ) then										-- Target Civ cannot DoW against itself

					if math.random(1, 100) < g_iAlliedReponseProbability then				-- Random # below threshold, allied civ will respond!						
						pTempTeam:DeclareWar(pPlayerTeam:GetID())							-- Force DOW											
						iDoWCivCount = iDoWCivCount + 1										-- Bump the DoW civ counter
						sTempString = sTempString.. pTempPlayer:GetCivilizationShortDescription() .. "   "	-- Append the civ to the running string
					end
				end
			end			
			if iDoWCivCount == 0 then														-- If no one DoWed, empty the string 			 
				sTempString = "There has been only a minor international murmur about this incident." 
			else
				--sTempString = sTempString .. "[NEWLINE]"
			end							
			NotificationStringLong = NotificationStringLong .. sTempString

		-- ======================		Response: Permanent War with target ===================
		--elseif iIndex == RESPONSE_WAR_PERMANENT then

		-- ======================		Response: Global War ===================
		elseif iIndex == RESPONSE_WAR_GLOBAL then											-- Possible DOW's by all known civs, with possibility of brief resistance in the capital
																							-- Note: different civs have different probabilities to DOW: DOF's have highest chance, Civs who have denounced Active civ have next highest, normal civs have lowest chance
			local pPlayerTeam = Teams[pPlayer:GetTeam()]									-- Get team pointer for atacking player
			local sTempString = "These Civilizations have declared war on " .. pPlayer:GetCivilizationShortDescription() .. ":  "			
			local iDenouncingCivs = 0
			local iTargetPlayerID = pTargetPlayer:GetID()									-- Get the target civ's team ID
			
			for idx = 1, GameDefines.MAX_MAJOR_CIVS-1, 1 do									-- Loop through all major AI players (skip human = 0)				
				local iDOW = NO
				local pTempPlayer = Players[idx]											-- Get the loop players instance
				local pTempTeam = Teams[pTempPlayer:GetTeam()]								-- Get team pointer for loop player

				if ( pTempPlayer:IsAlive() ) and											-- Only test for living civs
						( pTempTeam:IsHasMet(pPlayer:GetTeam()) ) and						-- Has loop civ met Active player
						( pPlayer:GetID() ~= idx ) then										-- And is the attacking civ NOT the target civ, then...
								
					if pTempPlayer:IsDoF(iTargetPlayerID) or (iTargetPlayerID == idx) then	-- Loop Civ has declared DOF with target civ or is target civ
						if math.random(1, 100) < g_iAlliedReponseProbability then			-- Random # below threshold, civ will respond!
							iDOW = YES
						end
				
					elseif pTempPlayer:IsDenouncingPlayer(pPlayer:GetID()) then			-- Has this player denounced Attacking player?  if so, then use delta between allies and all others' probabilities - split the difference
						if math.random(1, 100) < (g_iAlliedReponseProbability - g_iGlobalWarResponseProbability) then		-- Random # below threshold, civ will respond!
							iDOW = YES
						end
				
					else																	-- All other players use this probability
						if math.random(1, 100) < g_iGlobalWarResponseProbability then		-- Random # below threshold, civ will respond!
							iDOW = YES
						end
					end
					if iDOW == YES then														-- If any of the tests worked, start DOW						
						pTempTeam:DeclareWar(pPlayer:GetTeam())								-- Force DOW											
						iDenouncingCivs = iDenouncingCivs + 1								-- Bump the denouncing civ counter
						sTempString = sTempString.. pTempPlayer:GetCivilizationShortDescription() .. "   "	-- Add the civ to the running string
					end
				end
			end
			
			if iDenouncingCivs == 0 then													-- If no one denounced, empty the string 			 
				sTempString = "There has been only a minor international harumphing about this incident." 
			end							
			NotificationStringLong = NotificationStringLong .. sTempString

		else																				-- Should never reach here
			--print("End of Incidence Response without a valid iIndex, whose value is: " .. iIndex)
			NotificationStringLong = NotificationStringLong .. "This incident has caused an extensive international backlash!"	-- Indeterminate response, so issue a "safe" response
		end

		-- ==================================================================================================================================
		--		Domestic Turmoil?  
		local iDomesticTurmoilTurns = iIndex - 2											-- Potential "domestic turmoil" turns based on severity of international response (only happens for Denounce or higher, length 1..6 turns)
		if iDomesticTurmoilTurns < 1 then iDomesticTurmoilTurns = 0 end						-- If less than 1, there is no domestic response

		if iDomesticTurmoilTurns > 0 then													-- If we have a domestic response, then...
			local iDomesticDisapproval = CheckDomesticResponse(pPlayer, IncidentResult)		-- Determine if people are upset about incident; returns YES or NO; if at war, less chance of response
			if iDomesticDisapproval == YES then												-- If so, then
				local pCapitalCity = pPlayer:GetCapitalCity()								-- Get the capital city instance
				local iResistanceDuration = math.random(1, iDomesticTurmoilTurns)			-- Set duration of Resistance for 1..iDomesticTurmoilTurns  (should be 6 max)
				pCapitalCity:ChangeResistanceTurns(iResistanceDuration)						-- These resistance turns are cumulative!  This action adds turns to previous Resistance
				NotificationStringLong = NotificationStringLong .. "[NEWLINE][NEWLINE]The citizens of the " .. pPlayer:GetCivilizationAdjective() .. " capital are protesting their leadership (" .. iResistanceDuration .. " turns)."
			end	
		end
		
	else																					-- Mission response was "NO" 
		-- DO WE NEED An ELSE response?????????  "NO RESULT" = no report.  If at war, never see this so will always come to this else - to say what?  No response?  nothing happened?
		
		if pPlayer:IsHuman() then															-- Added better defualt return message, based on attacker (sman:20180211)
			NotificationStringLong = "Your operative escaped all implication in this affair."
		else
			NotificationStringLong = "The international community seems to have issued a collective yawn in response."
		end

	end
				
	--tMissionResultData[2] = tMissionResultData[2] .. "  " .. NotificationStringFading	-- Built return table data
	tMissionResultData[4] = NotificationStringLong
	tMissionResultData[6] = IncidentResult
end