-- Covert Operations Popup Functions
-- Author: sman
-- DateCreated: 12/21/17 1:21:51 PM
--------------------------------------------------------------

include("PlotMouseoverInclude.lua")

-- ===============================================================================================
--		Globals


-- ===============================================================================================
--		Support Functions

function SendNotification(iKey, NotificationStringLong, NotificationStringFading, iDisplayUnitType, x, y)		-- Displays notification for the right side of the screen, iKey parameter determines which notification displays
	if (iKey == 1) or (iKey == 2) or (iKey == 3) then										-- Prevent bad unitID from crashing notification
		if (not iDisplayUnitType) or (iDisplayUnitType < 0) then
			iKey = 4																		-- Use the generic blue spy icon for the notification
		end
	end
	
	if iKey == 1 then																		-- Display unit icon in a circle
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes.NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER, NotificationStringLong, NotificationStringFading,
			x, y, iDisplayUnitType, -1)

	elseif iKey == 2 then																	-- Display unit icon in animated blades/red bg
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes.NOTIFICATION_UNIT_DIED, NotificationStringLong, NotificationStringFading,
			x, y, iDisplayUnitType, -1)

	elseif iKey == 3 then																	-- Diplay unit icon with crosshairs and red bg
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes.NOTIFICATION_REBELS, NotificationStringLong, NotificationStringFading,
			x, y, iDisplayUnitType, -1)

	elseif iKey == 4 then																	-- Diplay blue spy icon: for AI Covert Ops Warnings; click on Notification icon brings camera to XY
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes.NOTIFICATION_SPY_RIG_ELECTION_SUCCESS, NotificationStringLong, NotificationStringFading,
			x, y)

	elseif iKey == 5 then																	-- Diplay red spy icon: for successful AI Covert Ops; click on Notification icon brings camera to XY 
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes.NOTIFICATION_SPY_WAS_KILLED, NotificationStringLong, NotificationStringFading,
			x, y)
	end
end


function GetRandomBuilding(targetCity)														-- Returns a random building type that exists in the city; no wonders returned!
	local randomBuildingType = -1															-- If no valid buildings exist, return nil
	local buildingsInCityTable = {}
	local buildingIDX = 0
	
	for row in GameInfo.Buildings() do														-- Scroll through all buildings in the Buildings table
		local buildnum =  targetCity:GetNumRealBuilding(row.ID)
		if targetCity:GetNumRealBuilding(row.ID) ~= 0  then									-- If the city has more than zero of these building types, then...
			local BClass = GameInfo.Buildings[row.ID].BuildingClass							-- Get the Building class of the building in the city
			local BWonder = GameInfo.BuildingClasses["" .. BClass .. ""].MaxGlobalInstances		-- Get MGI amount; normal buildings = 0; global wonders ~= 0
			local BNational = GameInfo.BuildingClasses["" .. BClass .. ""].MaxPlayerInstances	-- Get MPI amount; normal buildings = 0; national wonders ~= 0
			if (BWonder == -1) and (BNational == -1) then									-- If not a Global or National wonder then...
				buildingIDX = buildingIDX + 1												-- Increment the table index
				buildingsInCityTable[buildingIDX] = row.ID
			end
		end
	end
	if #buildingsInCityTable > 0  then														-- If there are one or more "normal" buildings in the city, then...
		local luckyBuilding = math.random(1, #buildingsInCityTable)							-- Generate random number between 1 and total count of normal buildings in city
		randomBuildingType = buildingsInCityTable[luckyBuilding]							-- Set return value to building ID
	end	
	return randomBuildingType																-- Return a existing normal building type or -1
end


function SpawnRebelUnits(targetCity, iSpawnCivID)											-- Spawns a number of units; type is the most advanced unit that civ has; spawn civ is the playerID num of the units	
	local pTargetPlayer = Players[targetCity:GetOwner()]									-- Get target city owner's player pointer
	local iTargetCivEra = pTargetPlayer:GetCurrentEra() or 2								-- Get that civ's Era, or 2 (Classical) as default
	local iUpperSpawnLimit = iTargetCivEra - 3												-- Limit spawn to Era-3, e.g. Classical=-1, Ren=0, Industrial=1, Modern=2, Atomic=3, Info=4
	if iUpperSpawnLimit < 1 then iUpperSpawnLimit = 1 end									-- But ensure at least 1
	if iUpperSpawnLimit > 3 then iUpperSpawnLimit = 4 end									-- And no more than 4 (just in case another mod adds Eras...)
							
	local spawnTotal = math.random(1, iUpperSpawnLimit)										-- Random number of Barb units to spawn; Ancient Era is 1..1, Infor Era is 1..5	
	local spawnUnitType = WARRIOR															-- Default partisan is a Warrior, should be changed later		
	local targetCityPlot = Map.GetPlot(targetCity:GetX(), targetCity:GetY())
	local freePlots = {}

	spawnUnitType = tRebelInfantryUnits [ iTargetCivEra ]									-- Spawn unit type = target city's civ's Era; most advanced unit that can spawn
	if not spawnUnitType then spawnUnitType = tRebelInfantryUnits [math.random(1, 2)] end	-- If some error getting spawnUnitType, then spawn Spear or Pikemen
		
	for iDX = -3, 3 do																		-- Search all ploys near target city: within x & y  +/- 3 tiles
		for iDY = -3, 3 do	
			local pTargetPlot = Map.PlotXYWithRangeCheck(targetCity:GetX(), targetCity:GetY(), iDX, iDY, 3)		-- Get the plot to examine					
			if pTargetPlot then																-- If plot is valid, then...
				local plotNumUnits = pTargetPlot:GetNumUnits()										-- Get the number of units on this plot
				if not (pTargetPlot:IsWater()) and not (plotNumUnits>0) and (pTargetPlot ~= targetCity:Plot() and not (pTargetPlot:IsMountain()) ) then	-- Only include empty land plots
					table.insert(freePlots, pTargetPlot)									-- Insert this plot into the freePlots table
				end
			end
		end
	end	
			
	for i = 1, #freePlots, 1 do																-- Shuffle the plot table, so units will appear at random locations surrounding targetCity
		local k = math.random(1, #freePlots)												-- Iterate through table sequentially, 1 .. #freePlots
		freePlots[i], freePlots[k] = freePlots[k], freePlots[i]								-- Swap the iterated plot with a random plot anywhere else in the table
	end
			
	if #freePlots < spawnTotal then spawnTotal = #freePlots end								-- Ensure no more units than valid plots spawn
	
	for n, plot in pairs(freePlots) do														-- Loop through the sorted plot table 							
		if n > spawnTotal then break end													-- Once n is more than the number of units to spawn, exit the loop early
			local partisanUnit = Players[iSpawnCivID]:InitUnit(spawnUnitType, plot:GetX(), plot:GetY())	-- spawn the iSpawnCivID civs unit 'spawnUnitType' at just outside target city
			partisanUnit:SetMoves(0)														-- Spawned units cannot move the turn they are created		
	end
	return spawnTotal, spawnUnitType
end


function GetBuildingInfluence(pTargetCity)													-- Returns the total influence of certain buildings if the city has them (currently = 0..53 possible)
	-- Defensive buildings only influence the chance of mission success and operator survival; does not impact international reaction
	local iInfluenceCount = 0																-- Initialize tally at 0	
	for i = 1, #g_tBuildingList do															-- Cycle through all the building types in the tBuildingList
		if pTargetCity:GetNumBuilding(g_tBuildingList[i][1]) > 0 then						-- Does this city have this building?
			iInfluenceCount = iInfluenceCount + g_tBuildingList[i][2]						-- If so, bump the tally by the influence amount [2]
			--print("city has bld type: " .. g_tBuildingList[i][1] .. "  raising influence by " .. g_tBuildingList[i][2] .. "  total is now: " .. iInfluenceCount)
		end
	end	
	return iInfluenceCount																	-- Return the tally as the total influence of all appropriate buildings in city
end


function GetUnitInfluence(pPlayer, pCity)													-- Returns the total influence of pPlayer's units in pCity's borders
	local iPPlayerID = pPlayer:GetID()														-- Get the pPlayer's ID
	local iUnitInfluence = 0																-- Tally of units in City tiles: normal unit=1, Covert Operator=CO level*3
	
	for i = 0, pCity:GetNumCityPlots() - 1, 1 do											-- Find open plots to spawn units on by examining all city owned plots
		local pWorkingPlot = pCity:GetCityIndexPlot( i )									-- Find the pointer to the current plot (i)
		
		if pWorkingPlot then																-- If valid plot, then...
			local iPlotUnitCount = pWorkingPlot:GetNumUnits()								-- Get the number of units on this plot		    
			for i = 0, iPlotUnitCount - 1, 1 do												-- Cycle through all units in the plot
    			local pFoundUnit = pWorkingPlot:GetUnit(i)									-- Get unit(i) pointer
				if (pFoundUnit:GetOwner() == iPPlayerID) then								-- If unit owner is the same as pPlayer (i.e. friendly unit), then				
					iUnitInfluence = iUnitInfluence + g_iUnitInfluenceMultiplier			-- Bump the tally by the unit's global influence amount (currently = 3)
					--print("Found a friendly unit. type: " .. pFoundUnit:GetName())
				end
			end
		end
	end	
	return iUnitInfluence
end


function GetRandomImprovement(pTargetCity)													-- Returns a Plot pointer if it finds a random improvement; nil otherwise
	local pReturnPlot = nil																	-- Return value is default nil
	local tImprovementTable = {}															-- Table of plot pointers		
	for i = 1, pTargetCity:GetNumCityPlots() - 1, 1 do										-- Find open plots to spawn units on by examining all city owned plots
		local pWorkingPlot = pTargetCity:GetCityIndexPlot( i )								-- Find the pointer to the current plot (i)		
		if pWorkingPlot then																-- If valid plot, then...		
			if not pWorkingPlot:IsCity() then												-- Don't add city plot to ImprovementTable; check all others			
				if pWorkingPlot:GetImprovementType() ~= -1 then								-- GetImprovementType returns -1 for no improvement				
					if not pWorkingPlot:IsImprovementPillaged() then						-- If the plot is improved and it is not pillaged, then...
						tImprovementTable[#tImprovementTable+1] = pWorkingPlot				-- Add the plot pointer to the table
					end
				end
			end
		end
	end
	if #tImprovementTable > 0 then															-- If there are any improved plots in the city, then...
		local iPlotIndex = math.random(1, #tImprovementTable)								-- Pick a random plot from the table
		pReturnPlot = tImprovementTable[iPlotIndex]											-- Adjust the return value to the selected plot's pointed
	end	
	return pReturnPlot
end


function GetRandomCity(pPlayer, bIsCapitalOK)												-- Returns a (semi) random city pointer in pPlayer's civ; if bIsCapitalOK = false, then return any non-capital city or nil if pPlayer only has 1 city
	-- Rules: 25% of the time, will return the capital, 25% of the time it will return the largest city (by population); 50% of the time a truly random city
	local iNumCities = pPlayer:GetNumCities() or 0											-- Track total number of pPlayer cities; default is 0	
	if iNumCities < 1 then																	-- If pPlayer has no cities (complete kills on?) return nil, leave early
		print("ERROR - player " .. pPlayer" .. has no cities!")
		return nil
	end

	local pCity = nil																		-- Can return nil if pPlayer has no cities or capital (complete kills on); should never happen from here, due to trap at top of function! 
	local iReturnCity = math.random(1, 100)													-- What kind of city do we return?
	
	if not bIsCapitalOK then iReturnCity = math.random(26, 100) end							-- If capital is NOT ok, then ensure either largest city or truly random city (largest city becomes more common by default: 33% chance instead of 25%)
	
	if iReturnCity < 25 then																-- Return the capital
		pCity = pPlayer:GetCapitalCity()													-- Set the return city pointer to the capital city pointer
		
	elseif iReturnCity < 50 then															-- Return the largest city, often the capital - which must be tested for if NOT bIsCapitalOK
		local iLargestPopulation = 0
		for iLoopCity in pPlayer:Cities() do												-- Cycle through all of pPlayer's cities
			if ( bIsCapitalOK ) or															-- If capitals are OK, then process all cities    OR
					( not bIsCapitalOK and not iLoopCity:IsCapital() ) then					-- If capitals are NOT OK and this city is not a capital, ok to process
				local iLoopCityPop = iLoopCity:GetPopulation()								-- Get the loop city pop
				if iLoopCityPop > iLargestPopulation then									-- If it's more than our previously largest city's pop, then
					iLargestPopulation = iLoopCityPop										-- Set the largest pop to loop city's pop
					pCity = iLoopCity														-- Set city pointer to loop city's pointed
				end
			end
		end																					-- At loop's end, we should have largest city's pointer; a nil should only happen if bIsCapitalOK is true and pPlayer only has 1 city	
	
	else																					-- Return any city in the civ
		local iCounter = 0																	-- Temp index
		local tAllowedCityTable = {}														-- Table of "allowed" cities to consider		
		for iLoopCity in pPlayer:Cities() do												-- Loop through all pPlayer's cities
			if ( bIsCapitalOK ) or															-- If capitals are OK, OR
					( not bIsCapitalOK and not iLoopCity:IsCapital() ) then					-- If capitals are NOT OK and this city is not a capital, then			
				iCounter = iCounter + 1														-- Increment the counter of loop iterations/total number of legal cities
				tAllowedCityTable[iCounter] = iLoopCity										-- Add this city pointer to the "allowed cities table"
			end
		end
		if #tAllowedCityTable >= 1 then														-- If one or more legal cities, then
			local iReturnCity = math.random(1, #tAllowedCityTable)							-- Random index from 1 .. Total num legal cities
			pCity = tAllowedCityTable[iReturnCity]											-- Set the return value to the city pointer in the corresponding table element
		end
	end
	return pCity																			-- Return the city pointer
end


function SpyInCityCheck(pPlayer, pCity)														-- Checks if a spy is located in pCity for pPlayer
	local bSpyInCity = false																-- Default return values are false/empty string/0
	local sSpyName = ""
	local iSpyRank = 1
	local sSpyMission = ""
	local iSpyPercentComplete = 0
	local iSpyTurnsLeft = 0

	if pPlayer and pCity then																-- Must have valid player and city pointers to accurately process
		local iCityX = pCity:GetX()															-- Need city X/Y to match spy location X/Y (can't match at the city pointer level, since there is not city pointer in the spy table
		local iCityY = pCity:GetY()
		local tSpyInfo = pPlayer:GetEspionageSpies()										-- Get Spy Table data for pPlayer		
		local sSpyCityName = "Unknown"

		local iSpyCount = #tSpyInfo or 0													-- Count rows in tSpyInfo for spy count	
		if iSpyCount > 0 then																-- If there's 1 or more spies, then...
			for i,v in ipairs(tSpyInfo) do													-- Cycle through all spies in the table		
				if (iCityX == v.CityX) and (iCityY == v.CityY) then							-- The Agent's location city XY matches pCity's XY, then...			
					bSpyInCity = true														-- Set bSpyInCity to true
					sSpyName = Locale.ConvertTextKey(v.Name) or "Unknown"					-- Get spy's name
					iSpyRank = g_SpyRankIndexValue[v.Rank] or 1								-- Get spy's rank, index based on text key (1..3)										
					sSpyMission = Locale.ConvertTextKey(v.State) or "Unknown"				-- Get spy's mission
					iSpyPercentComplete = v.PercentComplete or 0							-- Get spy's percent complete								
					iSpyTurnsLeft = v.TurnsLeft or 0										-- Turns left (to establish surveillence?)
					--print("============ >   Spy Found in " .. pCity:GetName() .. "    Owner: " .. pCity:GetOwner() .. "    Name: " .. sSpyName .. "    Rank: " .. sSpyRank .. "    Mission: " .. sSpyMission .. "    %Complete: " .. sSpyPercentComplete)
					break																	-- Found a spy in pCity belonging to pPlayer, exit loop early
				end			
			end	
		end
	end
	--if bSpyInCity then
	--	print("Player: " .. pPlayer:GetCivilizationShortDescription() .. " has a spy in " .. pCity:GetName() .. " - Name: " .. sSpyName .. "    Rank: " .. iSpyRank .. "    Mission: " .. sSpyMission .. "    % Complete: " .. sSpyPercentComplete)
	--else
	--	print("Player: " .. pPlayer:GetCivilizationShortDescription() .. " has no spies in " .. pCity:GetName() )
	--end
	return bSpyInCity, sSpyName, iSpyRank, sSpyMission, iSpyPercentComplete, iSpyTurnsLeft
end


-- returns pPlayer's IP for targetCity (pPlayer can be either attacker or owner)

function CalculateEspionagePosture(pPlayer, pCityInQuestion)								-- Returns the Esp. Posture or 0; based on a combination of defensive buildings in the target city and the player's capital
	local iEP = 0	
	local iBuildingInfluence = 0															-- Default return value
	local iSpyInfluence = 0	
	local iCityInQuestionOwnerID = pCityInQuestion:GetOwner()

	-- =======================================================================================================================================================
	-- Calculate Buildings Influence  (a number between 0..~53, depending on what buildings are in the city)
	if pPlayer:GetID() == pCityInQuestion:GetOwner()  then									-- pPlayer is defending - gets benefits of defensive buildings
		iBuildingInfluence = GetBuildingInfluence(pCityInQuestion) or 0						-- Determine building influence
	end	
	
	-- =======================================================================================================================================================
	-- Calculate Unit Influence  (~Number of units of all kinds near city * multiplier, which is currently 3.  So, Num units * 3
	local iUnitInfluence = GetUnitInfluence(pPlayer, pCityInQuestion) or 0					-- Determine unit influence for pPlayer in pCityInQuestion
			
	-- =======================================================================================================================================================
	-- Calculate Spy in City influcence		(Spy rank [1..3] OR 0 [for no spy] * 20.  So, range = 0..60)
	local bSpyInCity, sSpyName, iSpyRank, sSpyMission, iSpyPercentComplete, iSpyTurnsLeft = SpyInCityCheck(pPlayer, pCityInQuestion)		-- Check to see if friendly spy is in pTargetCity
	
	if bSpyInCity then iSpyInfluence = (iSpyRank * 20) or 20 end							-- Increases influence, based on spy ran, minimum should be 20 (Rank=1)
	
	-- Raw influence numbers: Buildings (53) + Units (xx) + Spy (60).  With 4 units near city, the number would be: 53+12+60 = 125 (absolute max to be expected in a spectacular capitla city)  Normal capital mid-game would be more like 20+6+20=46
			
	iEP = math.floor( (iBuildingInfluence + iUnitInfluence + iSpyInfluence) / 2 )			-- Reduce overall impact by half

	--print("Espionage Posture. Player: " .. pPlayer:GetCivilizationShortDescription() .. "    City: " .. pCityInQuestion:GetName() .. "    Building Influence: " .. iBuildingInfluence .. "    Unit Influence: " .. iUnitInfluence .. "    Spy Influence: " .. iSpyInfluence .. "    EP (*.5): " .. iEP)
	
	return iEP																				-- In a "perfect" capital, return=66; a normal mid-game capital=23
end


function GetAICivSpymaster(pPlayer)															-- Looking for a Spymaster, either the spy in the capital (counter-espionage) or a random spy in the spy table...
	local sSpymasterName = "Unknown"														-- Default if we don't find a spy (pPlayer in Ancient?_
	local sSpymasterRank = "TXT_KEY_SPY_RANK_0"												-- Default for no spy found
	local pCapitalCity = pPlayer:GetCapitalCity()											-- Get pointer for pPlayer's capital
	if pCapitalCity then																	-- Look in the Capital city first, "seniority..."
		local bSpyInCity, sSpyName, iSpyRank, sSpyMission, iSpyPercentComplete, iSpyTurnsLeft = SpyInCityCheck(pPlayer, pCapitalCity)
		if bSpyInCity then																	-- Pull the Spymaster from the capital's spy, if one is there
			sSpymasterName = sSpyName
			iSpymasterRank = iSpyRank
		else																				-- If not, grab a different one
			local tSpyInfo = pPlayer:GetEspionageSpies()									-- Get Spy Table data for pPlayer	
			local iSpyCount = #tSpyInfo or 0												-- Count rows in tSpyInfo for spy count	
			if iSpyCount > 0 then															-- If there's 1 or more spies, then...
				local iSpyMasterIDX = math.random(1, iSpyCount)
				sSpyName = tSpyInfo[iSpyMasterIDX].name
				iSpyRank = g_SpyRankIndexValue[tSpyInfo[iSpyMasterIDX].Rank] or 1			-- Return 1..3 or 1 number				
			end
		end
	end
	--print("==> AI Spymaster for " .. pPlayer:GetCivilizationShortDescription() .. " is " .. sSpymasterName .. " (" .. sSpymasterRank .. ")")	
	return sSpymasterName, iSpymasterRank
end


function GetUnitPurchaseCost(pPlayer, iPurchaseUnitID)										-- Returns the gold purchase price of iPurchaseUnitID	
	local pCapCity = pPlayer:GetCapitalCity()												-- Get the capital city for pPlayer 	
	local iPlayerEra = pPlayer:GetCapitalCity()												-- Get the capital city for pPlayer 	
	local iUnitPrice = -1																	-- Default price null
	--local sCityName = ""																	-- Default city name null
	if pCapCity then																		-- Of pPlayer has a capital, then...
		iUnitPrice = pCapCity:GetUnitPurchaseCost( iPurchaseUnitID )						-- Set the purchase price as if the capital bought it
		--sCityName = pCapCity:GetName()
	else																					-- No capital (complete kills?), so scroll through all cities (yea, I know, waste of time...)	
		for iLoopCity in pPlayer:Cities() do												-- Cycle through all of pPlayer's cities						
			iUnitPrice = iLoopCity:GetUnitPurchaseCost( iPurchaseUnitID )					-- Set the purchase price as if a different city bought it (should never happen!)
			--sCityName = iLoopCity:GetName()
			break																			-- Terminate loop early
		end	
	end
	if iUnitPrice == -1 then iUnitPrice = pPlayer:GetCurrentEra() * 125 end					-- If all else fails, give some kind of approximated cost
	--print("City name: " .. sCityName .. "   UnitID: " .. iPurchaseUnitID .. "    Price: " .. iUnitPrice)	
	return iUnitPrice	
end


function GetMissionCost(pPlayer, iMissionNumber, pTargetPlayer, pSelectedCity)				-- Builds the Mission cost, based on several factors
	if iMissionNumber == ABORT_MISSION then													-- It costs nothing to do nothing, return early
		return 0
	end
	
	local iTargetCityEra = pTargetPlayer:GetCurrentEra()									-- Get target city owner's current Era	
	local iHighestUnitType = tRebelInfantryUnits [ iTargetCityEra ]							-- What is the target city's most advanced infantry unit?
	local iUnitPurchaseCost = GetUnitPurchaseCost(pPlayer, iHighestUnitType)				-- Most advanced Infantry unit gold purchase cost: the "baseline" that missions are measured by (target city size too, added later)
	local iTargetCityPop = pSelectedCity:GetPopulation()									-- Get current target city pop - adjust mission costs, i.e. larger cities cost more to conduct a CO in			
	
	local fCityPopMultiplier = iTargetCityPop / 10											-- Divide the city pop by 10
	if fCityPopMultiplier < .03 then fCityPopMultiplier = .03 end							-- Don't let the cost get too small
	if fCityPopMultiplier > 3 then fCityPopMultiplier = 3 end								-- Don't let the costs grow too high

	local iMissionCost = math.floor(  (g_MissionList[iMissionNumber][ML_COST_FACTOR] * iUnitPurchaseCost ) * fCityPopMultiplier  )

	if not pPlayer:IsHuman() then iMissionCost = math.floor (iMissionCost / 2) end			-- Cut costs for AI missions in half!

	if iMissionCost < 0 then iMissionCost = 0 end											-- Make sure the cost is NLT 0 - for "Gold Steal" mission (sman: 20180209)
	--if iMissionCost < 10 then iMissionCost = 10 end											-- Make sure the cost is a non-trival (?) positive number
	
	return iMissionCost
end


function GetMostLikelyAIMission (pPlayer, iCurrentEra, iTargetPlayerID, pTargetCity)
	local iSelectedMission = 3																-- Default mission is "Spread Rumors"
	local iSelectedMissionCost = 0															-- Default mission cost is 0
	local pTargetPlayer = Players[iTargetPlayerID]											-- Get target player pointer
	local iAttackingPlayerGold = pPlayer:GetGold()											-- Get total gold target civ has on hand	
	
	local iRandom = NO																		-- Used to determine if mission was properly selected or random (for testing)
	local iTotalChance = 0																	-- The total percentage of mission chances of available missions (a sum of all availble mission chances)
	local tValidMissions = {}																-- Create table to hold list of affordable missions

	--print("Getting AI mission.  Player: " .. pPlayer:GetCivilizationShortDescription() .. "    GOH: " .. iAttackingPlayerGold)		

	for iNum = 2, ABORT_MISSION-1 do														-- Cycle through all missions, "skip Conduct Recon" and "abort mission"

		if (g_MissionList[iNum]) and														-- CHANGE: adjustment so when player disables missions, the loop works correctly (sman:20180210)
				(g_MissionList[iNum][ML_MISSION_AVAILABLE_ERA] <= iCurrentEra) then			-- If Mission #iNum's "level" is LTE to civ's current Era, allow it in the menu
				
			local iMissionCost = GetMissionCost(pPlayer, iNum, pTargetPlayer, pTargetCity)	-- Get the cost of the mission		
			if ( iMissionCost <= iAttackingPlayerGold ) then								-- If there's enough gold in the treasury, add it to the list
				tValidMissions[#tValidMissions + 1] = { [1] = iNum, [2] = iMissionCost, [3] = g_MissionList[iNum][ML_AI_CHANCE_TO_PICK] }	-- [ML_AI_CHANCE_TO_PICK] = AI chance to pick this
				iTotalChance = iTotalChance + g_MissionList[iNum][ML_AI_CHANCE_TO_PICK]
				--print("iNum: " .. iNum .. "    Mission: " .. g_MissionList[iNum][ML_MISSION_NAME].. "    Cost: " .. iMissionCost .. "    Chance: " .. g_MissionList[iNum][ML_AI_CHANCE_TO_PICK] .. "    Total Chance: " .. iTotalChance)				
			end
		end
	end

	if (#tValidMissions > 0) and 															-- Are there any possible missions?   Basically, if Steal Gold is all AI can afford, it's better to drop down to the random missions in the ELSE
			(( #tValidMissions == 1) and (tValidMissions[1][1] ~= STEAL_GOLD)) then			-- AND if only 1 mission available, and it's not Steal Gold, then....
	
		local iCorrectIndex = 1																-- Default mission is first one (least impacting)

		if #tValidMissions > 1 then															-- If there are more than 1 possible missions, then loop through them all		
			local iRunningPercentTally = 0													-- Zero out the total chance evaluated so far
			local iChanceResult = math.random(1, iTotalChance)								-- What is the chance for a mission?
			--print("iChanceResult: " .. iChanceResult)
			
			for idx = 1, #tValidMissions do													-- Look through all valid missions
				iRunningPercentTally = iRunningPercentTally + tValidMissions[idx][3]		-- Add chance of this mission (idx) to the total chances so far; [3] is AI percent chance of this mission
				if iChanceResult <= iRunningPercentTally then								-- If the chance for a mission is less than the running tally of mission chances, this is our mission
					iCorrectIndex = idx														-- Assign return mission to idx
					break																	-- exit loop early
				end
			end		
		end

		iSelectedMission = tValidMissions[iCorrectIndex][1]									-- Set return mission to the proper idx value
		iSelectedMissionCost = tValidMissions[iCorrectIndex][2]								-- Set mission cost to the proper idx value
		--print("Correct index: " .. iCorrectIndex .. "    Select mission: " .. iSelectedMission .. "    Cost: " .. iSelectedMissionCost)											
		
	else																					-- No mission was "properly" selected, so let's go random		
		local iMissionLevel = 1																-- Default random mission level is 1	(sman:20180211)
		if iCurrentEra > ERA_RENAISSANCE_ID then iMissionLevel = 2 end						-- If attacking civ is Industrial or later, random level is 2  

		iSelectedMission = GetRandomByLevelMission(2)										-- Find all the L1&2 missions enabled, pick a random one, or return -1
		if iSelectedMission == -1 then														-- If mission is invalid, then...
			iSelectedMissionCost = -1														-- Ensure the cost is invalid too
		else
			iSelectedMissionCost = GetMissionCost(pPlayer, iSelectedMission, pTargetPlayer, pTargetCity)	-- Othewise, get the valid mission cost
			iRandom = YES																	-- Test variable to flag that the assigned mission was random, not selected
		end
	end
	--print("AI Mission.  Player: " .. pPlayer:GetID() .. "    Mission: " .. iSelectedMission .. "    Cost: " .. iSelectedMissionCost .. "   Randomized? " .. iRandom)
	return iSelectedMission, iSelectedMissionCost
end


function GetRandomGreatPerson(pTargetCity)													-- Returns a GP unitID or -1 if no GP's near city
	local tGreatPersonList = {}
	local iLuckyGP = -1	
	local iTargetCivID = pTargetCity:GetOwner()												-- Get the pPlayer's ID	
	for i = 0, pTargetCity:GetNumCityPlots() - 1, 1 do										-- Find open plots to spawn units on by examining all city owned plots
		local pWorkingPlot = pTargetCity:GetCityIndexPlot( i )								-- Find the pointer to the current plot (i)
		local iPlotUnitCount = pWorkingPlot:GetNumUnits() or 0								-- Get the number of units on this plot		    
		for i = 0, iPlotUnitCount - 1, 1 do													-- Cycle through all units in the plot
    		local pFoundUnit = pWorkingPlot:GetUnit(i)										-- Get unit(i) pointer
			if (pFoundUnit:GetOwner() == iTargetCivID) and (pFoundUnit:IsGreatPerson()) then	-- If unit owner is the same as pPlayer (i.e. friendly unit), then								
				tGreatPersonList[#tGreatPersonList + 1] = pFoundUnit:GetID()
				--print("Adding GP: " .. pFoundUnit:GetName() .. "   Number: " .. #tGreatPersonList .. "    UnitID: " .. tGreatPersonList[#tGreatPersonList])
			end
		end
	end	
	if #tGreatPersonList > 0 then															-- If at least 1 GP is found, then...
		iLuckyGP = tGreatPersonList[math.random(1, #tGreatPersonList)]						-- Randomly select any GP from the table
	end	
	--print("Lucky GP ID: " .. iLuckyGP)
	return iLuckyGP
end


function GetRandomUnit(pTargetCity)															-- Returns a unitID of a random unit belonging to and near pTargetCity; returns -1 if no units around
	local tUnitList = {}
	local iLuckyUnit = -1	
	local iTargetCivID = pTargetCity:GetOwner()												-- Get the pPlayer's ID
	
	for i = 0, pTargetCity:GetNumCityPlots() - 1, 1 do										-- Find open plots to spawn units on by examining all city owned plots
		local pWorkingPlot = pTargetCity:GetCityIndexPlot( i )								-- Find the pointer to the current plot (i)
		local iPlotUnitCount = pWorkingPlot:GetNumUnits()									-- Get the number of units on this plot		    
		for i = 0, iPlotUnitCount - 1, 1 do													-- Cycle through all units in the plot
    		local pFoundUnit = pWorkingPlot:GetUnit(i)										-- Get unit(i) pointer
			if (pFoundUnit:GetOwner() == iTargetCivID) and (not pFoundUnit:IsGreatPerson()) then	-- If unit owner is the same as pPlayer (i.e. friendly unit), then								
				tUnitList[#tUnitList + 1] = pFoundUnit:GetID()								-- Save the unitID in the table
				--print("Adding unit: " .. pFoundUnit:GetName() .. "   Index: " .. #tUnitList .. "    UnitID: " .. tUnitList[#tUnitList])
			end
		end
	end	
	if #tUnitList > 0 then																	-- At lease 1 unit was found...
		iLuckyUnit = tUnitList[math.random(1, #tUnitList)]
	end	
	--print("Luck unitID: " .. iLuckyUnit)
	return iLuckyUnit																		-- Returning the unitID or -1
end


function GetRandomByLevelMission(iMissionLevel)												-- Returns a random mission at iMissionLevel or lower; changed to allow varying levels (sman:20180211)
	local tValidMissionList = {}
	local iRandomMission = -1
	for iNum = 2, ABORT_MISSION-1 do														-- Cycle through all missions, "skip Conduct Recon" and "abort mission"
		if ( g_MissionList[iNum] ) then			
			if g_MissionList[iNum][ML_MISSION_LEVEL] <= iMissionLevel then					-- Any active mission iMissionLevel or lower is ok
				tValidMissionList[#tValidMissionList + 1] = iNum							-- [ML_AI_CHANCE_TO_PICK] = AI chance to pick this				
				--print("Adding random mission.  iNum: " .. iNum .. "    Mission: " .. g_MissionList[iNum][ML_MISSION_NAME])				
			end
		end
	end
	if #tValidMissionList > 0 then															-- At lease 1 unit was found...
		iRandomMission = tValidMissionList[math.random(1, #tValidMissionList)]
	end
	--print("Random mission: " .. iRandomMission)
	return iRandomMission	
end


function SetNukeSuitcaseDamage(pTargetCity, bIsFalloutShelter)								-- Sets fallout and unit damage in the event of Nuke Suitcase mission
	local iNumTilesDamaged = 0																-- Return value, default 0
	local iNumUnitsKilled = 0																-- Return value, default 0
	local iUpperBound = 90																	-- Upper limit of HP loss
	local iLowerBound = 50																	-- Lower limit of HP loss
	if bIsFalloutShelter then 																-- Lower loss if Bomb Shelter in pTargetCity (~ half)
		iUpperBound = 50 
		iLowerBound = 25
	end									
	--print("Assigning Suitcase Nuke damage to " .. pTargetCity:GetName() .. "    Upper Bound: " .. iUpperBound .. "    Lower Bound: " .. iLowerBound)		
	local iPlotX = pTargetCity:GetX()
	local iPlotY = pTargetCity:GetY()
	for iDX = -2, 2 do
		for iDY = -2, 2 do
			local pWorkingPlot = Map.PlotXYWithRangeCheck(iPlotX, iPlotY, iDX, iDY, 2)
			if pWorkingPlot then
				-- ==== Process fallout
				if (not pWorkingPlot:IsWater()) and (not pWorkingPlot:IsCity()) then		-- No fallout in city or coast/ocean				
					if math.random(1, 100) < 20 then										-- Only 20% chance of fallout
						pWorkingPlot:SetFeatureType(FeatureTypes.FEATURE_FALLOUT, -1)
						iNumTilesDamaged = iNumTilesDamaged + 1
						--print("Fallout on plot " .. pWorkingPlot:GetX() .. " / " .. pWorkingPlot:GetY() .. "    Total tiles damaged: " .. iNumTilesDamaged)
					end
				end
				-- ==== Process unit damage
				local iPlotUnitCount = pWorkingPlot:GetNumUnits()									-- Get the number of units on this plot		    
				for i = 0, iPlotUnitCount - 1, 1 do													-- Cycle through all units in the plot
    				local pFoundUnit = pWorkingPlot:GetUnit(i)										-- Get unit(i) pointer
					local iUnitDamage = pFoundUnit:GetDamage() + math.random(iLowerBound, iUpperBound)
					pFoundUnit:SetDamage(iUnitDamage)
					if iUnitDamage >= pFoundUnit:GetMaxHitPoints() then iNumUnitsKilled = iNumUnitsKilled + 1 end
					--print("Unit: " .. pFoundUnit:GetName() .. "   New damage: " .. iUnitDamage .. "    Total #units killed: " .. iNumUnitsKilled)									
				end	
			end
		end
	end
	--print("#Tiles Damaged: " .. iNumTilesDamaged .. "    #Units Killed: " .. iNumUnitsKilled)
	return iNumTilesDamaged, iNumUnitsKilled	
end


--bIsGreatPersonNear = 
--function IsGreatPersonNear(pTargetCity, pTargetPlayer)
--end


--bIsUraniumAvailable = 
function FreeUraniumCount(pPlayer)
	local iFreeUraniumCount = pPlayer:GetNumResourceAvailable(RESOURCE_URANIUM_ID, true) or 0		-- Boolean = include imports
	--print("Player: " .. pPlayer:GetCivilizationShortDescription() .. " has " .. iFreeUraniumCount .. " free Uranium resources available")
	return iFreeUraniumCount
end


function BuildMissionMenu(pPlayer, pTargetCity)												-- Returns a table of valid mission numbers for this player vs this target city; assumes bSpyIsEstablished check is performed before being called
	local tMissionTable = {}																-- Default nil table if no valid missions; usually should be at least 1 = ABORT
	local iCurrentEra = pPlayer:GetCurrentEra()	or 0
	local iTargetCityOwnerID = pTargetCity:GetOwner()
	local pTargetPlayer = Players[iTargetCityOwnerID]
	local iAttackingPlayerGold = pPlayer:GetGold()											-- Get total gold target civ has on hand	
	
	if pPlayer:GetID() == iTargetCityOwnerID then											-- Return early if pPlayer owns targetcity
		print("BuildMissionMenu Error.  Attacking Civ: " .. pPlayer:GetCivilizationShortDescription() .. " is the owner of " .. pTargetCity:GetName())
		return tMissionTable
	end
		
	local bIsHumanAttacker = true															-- Default to human player, with all missions
	local iStartLoop = CONDUCT_RECON
	local iStopLoop = ABORT_MISSION															-- Don't add ABORT to valid mission table, even for human player
	if not pPlayer:IsHuman() then															-- If pPlayer is AI, change start missions away from RECON; NYI
		bIsHumanAttacker = false
		iStartLoop = CONDUCT_RECON + 1														
		iStopLoop = ABORT_MISSION	-1	
	end
	
	local bSelectedCityIsCapital = false													-- Is target city a capital?, default to false
	if pTargetCity:IsCapital() then bSelectedCityIsCapital = true end						-- If target city is capital, set flag to true (true for City States, too BTW)
	local bSelectedCityIsCityState = false													-- Is target city a city state?, default to false
	if pTargetPlayer:IsMinorCiv() then bSelectedCityIsCityState = true end					-- If target city is city state, set flag to true
	
	local bIsGreatPersonNear = false														-- Is a GP is near the city?
	if GetRandomGreatPerson(pTargetCity) > -1 then bIsGreatPersonNear = true end	

	local bIsUraniumAvailable = false														-- Does this civ have at least 1 free Uranium?
	if FreeUraniumCount(pPlayer) > 0 then bIsUraniumAvailable = true end
	
	for iNum = iStartLoop, iStopLoop do														-- Cycle through all missions (1..N for humans, 2..N-1 for AI)			
		if (g_MissionList[iNum]) and														-- Ensure no one disabled this mission
				(g_MissionList[iNum][ML_MISSION_AVAILABLE_ERA] <= iCurrentEra) then			-- If Mission #iNum's "level" is LTE to pPlayer's current Era, allow it in the menu
				
			-- === Qualify the mission based on city:  Yes for all missions on a normal city; check for Capital or City State eligible missions
			if  (not bSelectedCityIsCapital) or													-- For non-capital cities, all missions valid, then...	
					((g_MissionList[iNum][ML_VALID_CAPITAL] == YES) and (bSelectedCityIsCapital) and (not bSelectedCityIsCityState))  or	-- For capital city: mission is valid, city is cap, city is not city state, then
					((g_MissionList[iNum][ML_VALID_CITYSTATE] == YES) and (bSelectedCityIsCityState))  then		-- For city state: mission is valid, city is city state, then...
					
				-- === Qualify individual missions: Most missions will become qualified by default later on - in this IF's "else"
				local bIsMissionValid = false												-- Assume the mission isn't valid unless changed later								
				--local iMissionCost = GetMissionCost(pPlayer, iNum, pTargetPlayer, pTargetCity)
				

				-- === Missions with specific requirements	=================================================================================
						
				if ((iNum == POISON_WATER) and (pTargetCity:GetPopulation() > 1)) then		-- No need to attack a city with 1 pop
					bIsMissionValid = true
					
				elseif (iNum == SABOTAGE_UNIT) then
					local iTempInfluence = GetUnitInfluence(pTargetPlayer, pTargetCity)		-- Returns > 0 if any units present, good approximate to determine to add or not
					if iTempInfluence > 0 then bIsMissionValid = true end
					
				elseif (iNum == PILLAGE_IMPROVEMENT) then
					local pTempImprovemnet = GetRandomImprovement(pTargetCity)				-- Returns non-nil if any units present, good approximate to determine to add or not
					if pTempImprovemnet then bIsMissionValid = true end
					
				elseif ((iNum == ASSASSINATE_GREAT_PERSON) and (bIsGreatPersonNear)) then	-- No need to try mission if no GPs around					
					bIsMissionValid = true

				elseif ((iNum == NUKE_SUITCASE) and (bIsUraniumAvailable)) then				-- Cannot attempt without spare uranium
					bIsMissionValid = true


				-- === Allow missions with NO special requirements	===========================================================================
				elseif	(iNum == CONDUCT_RECON) or (iNum == SPREAD_RUMORS) or (iNum == SABOTAGE_PRODUCTION) or (iNum == STEAL_GOLD) or
						(iNum == SPREAD_PLAGUE) or (iNum == DESTROY_BUILDING) or (iNum == RECRUIT_PARTISANS) or (iNum == SPREAD_PROPAGANDA) or 
						(iNum == CAUSE_RESISTANCE) or (iNum == CAUSE_ANARCHY) or (iNum == STEAL_CITY) or (iNum == ABORT_MISSION) then

					bIsMissionValid = true													-- With no specific restrictions, allow mission by default

				end
				
				local iMissionCost = GetMissionCost(pPlayer, iNum, pTargetPlayer, pTargetCity)	-- Get cost of mission - probably only applies to human player for now					
				if ((bIsHumanAttacker) and (bIsMissionValid)) then							-- We know if mission iNum is "technically" valid, but for humans...
					if ( iMissionCost <= iAttackingPlayerGold ) then						-- Does the human player have enough gold to launch?
						tMissionTable[#tMissionTable + 1] = iNum							-- If so, add this mission to the table
					end

				else
					-- Need to work to add AI to this - not sure it's a good fit yet...
				end
			end
		end
	end
	--print("BuildMissionTable.  Player: " .. pPlayer:GetCivilizationShortDescription()  .. "    Target City: " .. pTargetCity:GetName() .. "    Number of missions: " .. #tMissionTable)
	return tMissionTable
end




function NumValidMissionsWorldwide(pPlayer)													-- Returns Number of missions available worldwide (in foreign cities); non-ABORT missions, that is
																							-- Used to determine if Notification should be displayed or not, even if pPlayer is eligible for a mission
																							-- For human player only
	--local bAnyMissionsAnywhere = false														-- Default return is false
	local iTotalMissionCount = 0

	local tSpyInfo = pPlayer:GetEspionageSpies()									-- Get Spy Table data for pPlayer	
	for iDX = 1, #tSpyInfo do		
		local pTempPlot = Map.GetPlot(tSpyInfo[iDX].CityX, tSpyInfo[iDX].CityY);
		local pTempCity = nil
		if(pTempPlot ~= nil) then
			pTempCity = pTempPlot:GetPlotCity()
			if ( (pTempCity ~= nil) and (pPlayer:GetID() ~= pTempCity:GetOwner()) and (IsSpyEstablished(pPlayer, pTempCity)) ) then	-- City must be valid and foreign and spy is established
				local tTempMissionTable = BuildMissionMenu(pPlayer, pTempCity)				-- How many missions can this spy perform?
				if #tTempMissionTable > 1 then
					--print("City: " .. pTempCity:GetName() .. "    Found " .. (#tTempMissionTable - 1) .. " possible missions available")
					iTotalMissionCount = iTotalMissionCount + (#tTempMissionTable - 1)		-- Tally number of non-ABORT missions					
				end
			end
		end					
	end
	--print("Player: " .. pPlayer:GetCivilizationShortDescription() .. " has " .. iTotalMissionCount .. " worldwide missions possible")
	return iTotalMissionCount
end


function IsSpyEstablished(pPlayer, pTargetCity)
	local bIsSpyEstablished = false
	local sAgentActivity = ""
	-- test for spy activity to see if he can legally launch CO

	local bSpyInCity, sSpyName, iSpyRank, sSpyMission, iSpyPercentComplete, iSpyTurnsLeft = SpyInCityCheck(pPlayer, pTargetCity)	-- Check to see if friendly spy is in pTargetCity

	--print("IsSpyEstablished. City: " .. pTargetCity:GetName() .. "    Name: " .. sSpyName .. "   Mission/State: " .. sSpyMission .. "    Turns Left: " .. iSpyTurnsLeft)
	
	if (sSpyMission == Locale.Lookup("TXT_KEY_SPY_STATE_SURVEILLANCE") and iSpyTurnsLeft < 0) then
		sAgentActivity = Locale.Lookup("TXT_KEY_SPY_STATE_ESTABLISHED_SURVEILLANCE")
	else
		sAgentActivity = Locale.Lookup(sSpyMission)	
	end		
	
	if (sAgentActivity == Locale.Lookup("TXT_KEY_SPY_STATE_GATHERING_INTEL")) or									-- Spy is gathering intel
				(sAgentActivity == Locale.Lookup("TXT_KEY_SPY_STATE_RIGGING_ELECTION")) or							-- Spy is rigging election
				(sAgentActivity == Locale.Lookup("TXT_KEY_SPY_STATE_SCHMOOZING")) or								-- Diplomat is Schmoozing 				
				(sAgentActivity == Locale.Lookup("TXT_KEY_SPY_STATE_ESTABLISHED_SURVEILLANCE")) or					-- Spy has established surveillance
				(sAgentActivity == Locale.Lookup("TXT_KEY_SPY_STATE_ESTABLISHED_SURVEILLANCE_PROGRESS_BAR")) then	-- Spy has no techs to steal
			
		bIsSpyEstablished = true
		--print("Spy IS established in " .. pTargetCity:GetName())
	end

	return bIsSpyEstablished
end










function SleepInSeconds(iSeconds)
  local ntime = os.time() + iSeconds
  repeat until os.time() > ntime
end